/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  drv884x.h - Declaration file for utility functions and global variables
 *  DRV884x_EVM_FIRMWARE
 *  08/25/2022
 *  Jacob Thompson
 *
 ******************************************************************************/
#ifndef DRV8845_H_
#define DRV8845_H_
/******************************************************************************
 * HEADER FILES
 *****************************************************************************/
#include "msp430.h"
#include "dataTypeDefinition.h"

/******************************************************************************
 * MACROS
 *****************************************************************************/

// DRV8845  "Enable" Pins - made by toggling I0x and I01 together.  
// Note - pins I0x and I1x are already tied together in hardware with jumpers in DRV8845.  I01 and I11 are together.  I02 and I12 together. 
#define read_I01_I11_pin               P1IN & I01
#define set_I01_I11_lo                 P1OUT &= ~I01
#define set_I01_I11_hi                 P1OUT |= I01
#define read_I02_I12_pin               P1IN & I02
#define set_I02_I12_lo                 P1OUT &= ~I02
#define set_I02_I12_hi                 P1OUT |= I02
#define read_I03_I13_pin               P4IN & I03
#define set_I03_I13_lo                 P4OUT &= ~I03
#define set_I03_I13_hi                 P4OUT |= I03
#define read_I04_I14_pin               P4IN & I04
#define set_I04_I14_lo                 P4OUT &= ~I04
#define set_I04_I14_hi                 P4OUT |= I04

// Phase (DRV8845 only)
#define set_phase1_lo                  P1OUT &= ~PHASE1
#define set_phase2_lo                  P1OUT &= ~PHASE2
#define set_phase3_lo                  P4OUT &= ~PHASE3
#define set_phase4_lo                  P4OUT &= ~PHASE4
#define set_phase1_hi                  P1OUT |= PHASE1
#define set_phase2_hi                  P1OUT |= PHASE2
#define set_phase3_hi                  P4OUT |= PHASE3
#define set_phase4_hi                  P4OUT |= PHASE4
#define set_phase1_hiz                 P1DIR &= ~PHASE1
#define set_phase2_hiz                 P1DIR &= ~PHASE2
#define set_phase3_hiz                 P4DIR &= ~PHASE3
#define set_phase4_hiz                 P4DIR &= ~PHASE4

/******************************************************************************
* DATA STRUCTURES
 *****************************************************************************/

// Ramp duty controller
typedef struct DRV8845_RMPCNTL
{
  volatile uint8_t rampDelayMax;       // Parameter: Maximum delay rate (0-250)
  volatile uint8_t rampDelayCount;     // Variable: Incremental delay (0-250)
  volatile uint8_t setpointValue;      // Output: Target output(0-100)
} RMPCNTL_t;

// Brushed motor only parameters (DRV8845 on Brushed MotorMode)
typedef struct DRV8845_Device_Obj_t
{
  // Device Parameters for brushed DRV8845 (5 bytes + dummy)
  volatile uint8_t pwmFreq;            // 0: 25 kHz, 1: 50 kHz, 2: 100 kHz and 3: 200 kHz
  volatile uint8_t StartDrive1;
  volatile uint8_t StartDrive2;
  volatile uint8_t StartDrive3;
  volatile uint8_t StartDrive4;
  volatile uint8_t dummy1;             // for a zero fill byte to align memory location to even address in MSP430

  // Ramp controller (12 bytes) (3 bytes each)
  RMPCNTL_t rampInput1;
  RMPCNTL_t rampInput2;
  RMPCNTL_t rampInput3;
  RMPCNTL_t rampInput4;

  // Control Parameters (17 bytes)
  volatile uint8_t in1;                // in1 PWM duty cycle (DRV8845 Phase1)
  volatile uint8_t in2;                // in2 PWM duty cycle (DRV8845 Phase2)
  volatile uint8_t in3;                // in3 PWM duty cycle (DRV8845 Phase3)
  volatile uint8_t in4;                // in4 PWM duty cycle (DRV8845 Phase4)
  volatile uint8_t en1;                // enable in1 for DRV8845, made by I01 and I11 changing together
  volatile uint8_t en2;                // enable in2 for DRV8845, made by I02 and I12 changing together
  volatile uint8_t en3;                // enable in3 for DRV8845, made by I03 and I13 changing together
  volatile uint8_t en4;                // enable in4 for DRV8845, made by I04 and I14 changing together
  volatile uint8_t dir1;               // direction for Bridge 1 (Forward, Coast, Reverse)
  volatile uint8_t pre_dir1;           // previous direction for Bridge 1
  volatile uint8_t dir2;               // direction for Bridge 2
  volatile uint8_t pre_dir2;           // previous direction for Bridge 2
  volatile uint8_t dir3;               // direction for Bridge 3
  volatile uint8_t pre_dir3;           // previous direction for Bridge 3
  volatile uint8_t dir4;               // direction for Bridge 4
  volatile uint8_t pre_dir4;           // previous direction for Bridge 4


  // 32-bit Floating point variables (8 bytes)
  volatile float VREF1;                // Reference voltage for Full-Bridge 1, and 3 for DRV8845
  volatile float VREF2;                // Reference voltage for Full-Bridge 2, and 4 for DRV8845
                                       // 42 bytes total:  5 + 12 + 17 + 8 = 42
} DRV8845_Device_Obj_t;

// Utility variables for DRV8845 Brushed
volatile unsigned char interval_tick;
volatile unsigned char interval_count;
volatile unsigned char pwmState1;
volatile unsigned char pwmState2;
volatile unsigned char pwmState3;
volatile unsigned char pwmState4;
volatile float VREF1_previous;
volatile float VREF2_previous;
volatile uint8_t MotorModePrevious2;
volatile uint8_t prev_pwmFreq;


/******************************************************************************
 * EXTERN DECLARATIONS
 *****************************************************************************/
extern volatile DRV8845_Device_Obj_t gBrushed_Obj;

/******************************************************************************
 * API Prototypes
 *****************************************************************************/

void brushedTimerConfig(void);
void brushed_init(void);
void UpdateDeviceControls_Brushed_DRV8845(void);
void drv_PH_EN_RampControl(void);
void drv_RampCTRL(volatile RMPCNTL_t* rmpCntl, uint8_t targetValue);

/*****************************************************************************/
#endif // DRV884x_H_
