/* --COPYRIG  HT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/********************************************************************************
 *
 *  drv884x.c - Functions specific do DRV8845
 *  DRV884x_EVM_FIRMWARE
 *  01/03/2023
 *  Jacob Thompson
 *
 *******************************************************************************/


/********************************************************************************
 * HEADER FILES
 *******************************************************************************/
#include "drv884x/include/drv8845.h"
#include "drv884x/include/utility.h"
#include "stepper.h"
#include "stepper_8845.h"
#include "uart.h"

/********************************************************************************
 * Data Structures exported for GUI Composer
 *******************************************************************************/
#pragma DATA_SECTION(gBrushed_Obj,".bruVarSection");
#pragma DATA_SECTION(gDRV884x_Obj,".drvVarSection");

volatile DRV884x_Device_Obj_t gDRV884x_Obj;
volatile DRV8845_Device_Obj_t gBrushed_Obj;

/********************************************************************************
 * LOCAL ROUTINES
 *******************************************************************************/
// Timer Initialization for PWM outputs to driver
void brushedTimerConfig(void)
{
    // Initialize Timer B0 with 0% duty for IN1, IN2, IN3 and IN4
    TB0CTL = TBSSEL_2 | MC_0 | TBCLR;                                // Timer clock is SMCLK 20MHz, timer halted and cleared
    TB0CCR0 = PWM_TIMER_PERIOD;                                      // Timer period value

    TA0CTL = TASSEL_2 | MC_0 | TACLR;                                // Timer clock is SMCLK 20MHz, timer halted and cleared
    TA0CCR0 = PWM_TIMER_PERIOD;                                      // Timer period value
    TA0CCTL0 |= CCIE;                                                // Enable Timer A0 CCR0 interrupt

    TA0CCTL2 |= OUTMOD_3;                                            // Timer mode to PWM set/reset for I01/I11
    TA0CCTL4 |= OUTMOD_3;                                            // Timer mode to PWM set/reset for I02/I12
    TB0CCTL2 |= OUTMOD_3;                                            // Timer mode to PWM set/reset for I03/I13
    TB0CCTL4 |= OUTMOD_3;                                            // Timer mode to PWM set/reset for I04/I14

    TA0CCR2 = 0;                                                     // IN1 compare register reset
    TA0CCR4 = 0;                                                     // IN2 compare register reset
    TB0CCR2 = 0;                                                     // IN3 compare register reset
    TB0CCR4 = 0;                                                     // IN4 compare register reset

    TB0CTL = TBSSEL_2 | PWM_FREQ_25KHZ | MC_1 | TBCLR;               // Timer clock is SMCLK 20MHz, up mode, cleared to start counting from zero
    TA0CTL = TASSEL_2 | PWM_FREQ_25KHZ | MC_1 | TACLR;
    
    P1SEL |= I01 + I02;                                              // IN1, IN2 setup as timer PWM outputs
    P4SEL |= I03 + I04;                                              // IN3 and IN4 setup as timer PWM outputs
}

/*******************************************************************************
 * API IMPLEMENTATION
 *******************************************************************************/

// Initialize brushed drivers GUI variables
void brushed_init()
{
    // Reset Timer B0 and A2 from DRV8845 Stepper setup
    TB0CTL &= ~0xFFFF;                                               // Reset TB0 configuration
    TB0CCTL0 &= 0xFDF7;                                              // Reset TB0CCTL0
    TB0CCR0 = 0;                                                     // Clear TBOCCR0

    TA0CTL &= ~0xFFFF;                                               // Reset TA0 configuration
    TA0CCTL0 &= 0xF9F7;                                              // Reset TA0CCTL0
    TA0CCR0 = 0;                                                     // Clear TAOCCR0

    TA1CTL &= ~0xFFFF;                                               // Reset TA1 configuration
    TA1CCTL0 &= 0xF9F7;                                              // Reset TA1CCTL0
    TA1CCTL2 &= 0xF9F7;                                              // Reset TA1CCTL2
    TA1CCR0 = 0;                                                     // Clear TA1CCR0

    TA2CTL &= ~0xFFFF;                                               // Reset TA2 configuration
    TA2CCTL0 &= 0xF9F7;                                              // Reset TA2CCTL0
    TA2CCR0 = 0;                                                     // Clear TA2CCR0

    // Release pins from Stepper PWM output settings
    P1SEL &= ~STEP1;
    P4SEL &= ~STEP2;

    // REF configuration for 1.5V VREF for DAC12_x
    REFCTL0 |= REFON;                                                // Internal reference VREF on

    // DAC12 configuration
    DAC12_0CTL0 =  DAC12CALON | DAC12SREF_0 | DAC12IR | DAC12AMP_7;  // Configure with VREF=1.5V DAC12_0 on P6.6 initiate DAC12_1 calibration
    while (DAC12_0CTL0 & DAC12CALON);                                // Wait for Calibration complete
    DAC12_0DAT = 0;                                                  // Default output 0V

    DAC12_1CTL0 =  DAC12CALON | DAC12SREF_0 | DAC12IR | DAC12AMP_7;  // Configure with VREF=1.5V DAC12_1 on P6.7 initiate DAC12_1 calibration
    while (DAC12_1CTL0 & DAC12CALON);                                // Wait for Calibration complete
    DAC12_1DAT = 0;                                                  // Default output 0V
    //End DAC12 configuration

    // Global variables initialization
    gBrushed_Obj.pwmFreq = 4;
    gBrushed_Obj.StartDrive1 = 0;
    gBrushed_Obj.StartDrive2 = 0;
    gBrushed_Obj.StartDrive3 = 0;
    gBrushed_Obj.StartDrive4 = 0;
    gBrushed_Obj.in1 = 0;
    gBrushed_Obj.in2 = 0;
    gBrushed_Obj.in3 = 0;
    gBrushed_Obj.in4 = 0;
    gBrushed_Obj.en1 = 0;
    gBrushed_Obj.en2 = 0;
    gBrushed_Obj.en3 = 0;
    gBrushed_Obj.en4 = 0;
    gBrushed_Obj.dir1 = 1;
    gBrushed_Obj.pre_dir1 = 0;
    gBrushed_Obj.dir2 = 1;
    gBrushed_Obj.pre_dir2 = 0;
    gBrushed_Obj.dir3 = 1;
    gBrushed_Obj.pre_dir3 = 0;
    gBrushed_Obj.dir4 = 1;
    gBrushed_Obj.pre_dir4 = 0;
    gBrushed_Obj.VREF1 = 0.5;
    gBrushed_Obj.VREF2 = 0.5;
    VREF1_previous = 0;
    VREF2_previous = 0;

    pwmState1 = 0;
    pwmState2 = 0;
    pwmState3 = 0;
    pwmState4 = 0;
    prev_pwmFreq = 4;

    // Ramp controller init for in1
    gBrushed_Obj.rampInput1.rampDelayMax   = 50;
    gBrushed_Obj.rampInput1.rampDelayCount = 0;
    gBrushed_Obj.rampInput1.setpointValue  = 0;

    // Ramp controller init for in2
    gBrushed_Obj.rampInput2.rampDelayMax = 50;
    gBrushed_Obj.rampInput2.rampDelayCount = 0;
    gBrushed_Obj.rampInput2.setpointValue = 0;

    // Ramp controller init for in3
    gBrushed_Obj.rampInput3.rampDelayMax   = 50;
    gBrushed_Obj.rampInput3.rampDelayCount = 0;
    gBrushed_Obj.rampInput3.setpointValue  = 0;

    // Ramp controller init for in4
    gBrushed_Obj.rampInput4.rampDelayMax = 50;
    gBrushed_Obj.rampInput4.rampDelayCount = 0;
    gBrushed_Obj.rampInput4.setpointValue = 0;

    // Set outputs to Coast (I0x = I1x = HIGH)
    set_I01_I11_hi;
    set_I02_I12_hi;
    set_I03_I13_hi;
    set_I04_I14_hi;

}

// Read the GUI settings and update device control
void UpdateDeviceControls_Brushed_DRV8845()
{
    // toggle MCU_STATUS LED if count is reached
    if (LED_TOGGLE_CNT > LED_TIMEOUT)
    {
        toggle_MCUstatus_led;
        LED_TOGGLE_CNT = 0;

        // Measure VM and update to GUI display
        ADC12CTL0 = ADC12ON;                                         // Enable ADC
        ADC12IE = 0;                                                 // Disable ADC interrupts
        ADC12CTL1 = ADC12SHP;                                        // configure for single conversion
        ADC12MCTL0 = ADC12INCH_0 + ADC12SREF_0;                      // A0 for VM measurement
        ADC12CTL0 |= ADC12SC + ADC12ENC;                             // Enable and start conversion
        while (ADC12CTL1 & ADC12BUSY);                               // Wait for conversion to complete
        gDRV884x_Obj.VMvoltage = ADC12MEM0;                          // Conversion result stored in ADC12MEM0 sent to GUI display
        if (gDRV884x_Obj.VMvoltage < ADC12scale)
            gDRV884x_Obj.VMvoltage = 0;                              // VM is 0
        ADC12CTL0 &= ~ADC12ENC;                                      // Disable ADC
        // End measure VM and update to GUI display
    }

    // nSLEEP control
    if (gDRV884x_Obj.nSleep && !(read_nSLEEP_pin))
    {
        set_nSLEEP_hi;
        _delay_cycles(30000);                                        // 1.5 ms wake up delay

        // start the timers
        TB0CTL |= MC_1;
        TA0CTL |= MC_1;
    }
    else if (!gDRV884x_Obj.nSleep && (read_nSLEEP_pin))
    {
        gBrushed_Obj.rampInput1.setpointValue = 0;
        gBrushed_Obj.rampInput2.setpointValue = 0;
        gBrushed_Obj.rampInput3.setpointValue = 0;
        gBrushed_Obj.rampInput4.setpointValue = 0;
        gBrushed_Obj.in1 = 0;
        gBrushed_Obj.in2 = 0;
        gBrushed_Obj.in3 = 0;
        gBrushed_Obj.in4 = 0;
        gBrushed_Obj.en1 = 0;
        gBrushed_Obj.en2 = 0;
        gBrushed_Obj.en3 = 0;
        gBrushed_Obj.en4 = 0;
        __delay_cycles(100000);

        // stop the timers and clear count
        TB0CTL &= ~(MC_3 + TACLR);
        TA0CTL &= ~(MC_3 + TACLR);

        set_nSLEEP_lo;
    }

    // Clear Fault using nSLEEP reset pulse
    if (gDRV884x_Obj.ClrFlt)
    {
        ClearFaultPulse();
        gDRV884x_Obj.ClrFlt = 0;
    }
    
    // VREF1 and VREF3 update
    if (gBrushed_Obj.VREF1 != VREF1_previous)
    {
        DAC12_0DAT = 2730*gBrushed_Obj.VREF1;
        VREF1_previous = gBrushed_Obj.VREF1;
        _nop();
    }

    // VREF2 and VREF4 update
    if (gBrushed_Obj.VREF2 != VREF2_previous)
    {
        DAC12_1DAT = 2730*gBrushed_Obj.VREF2;
        VREF2_previous = gBrushed_Obj.VREF2;
        _nop();
    }
    
    //Start Motor 1 is pressed, ramp up PWM
    if (gBrushed_Obj.StartDrive1 && !pwmState1)
    {   
        pwmState1 = 1;
        // No enable LED
    }
    //Stop Motor 1 is pressed, ramp down PWM
    if (!gBrushed_Obj.StartDrive1 && pwmState1)
    {
        pwmState1 = 0;
        gBrushed_Obj.in1 = 0;
        // No enable LED
    } 

    //Start Motor 2 is pressed, ramp up PWM
    if (gBrushed_Obj.StartDrive2 && !pwmState2)
    {   
        pwmState2 = 1;
    }
    //Stop Motor 2 is pressed, ramp down PWM
    if (!gBrushed_Obj.StartDrive2 && pwmState2)
    {
        pwmState2 = 0;
        gBrushed_Obj.in2 = 0;
    } 

    //Start Motor 3 is pressed, ramp up PWM
    if (gBrushed_Obj.StartDrive3 && !pwmState3)
    {   
        pwmState3 = 1;
    }
    //Stop Motor 3 is pressed, ramp down PWM
    if (!gBrushed_Obj.StartDrive3 && pwmState3)
    {
        pwmState3 = 0;
        gBrushed_Obj.in3 = 0;
    } 

    //Start Motor 4 is pressed, ramp up PWM
    if (gBrushed_Obj.StartDrive4 && !pwmState4)
    {   
        pwmState4 = 1;
    }
    //Stop Motor 4 is pressed, ramp down PWM
    if (!gBrushed_Obj.StartDrive4 && pwmState4)
    {
        pwmState4 = 0;
        gBrushed_Obj.in4 = 0;
    } 

    //Select PWM frequency 200 kHz, 100 kHz, 50 kHz, 25 kHz, 12.5 kHz, 6.25 kHz, 5 kHz, and 3.125 kHz
    if (!gBrushed_Obj.StartDrive1 && !gBrushed_Obj.StartDrive2 && !gBrushed_Obj.StartDrive3 && !gBrushed_Obj.StartDrive4
        && prev_pwmFreq != gBrushed_Obj.pwmFreq)
    {
        switch (gBrushed_Obj.pwmFreq)
        {
        case 0:
            TB0CTL    |= ID_3 | TBCLR;
            TB0EX0     = TBIDEX__8;                                  // 2.5 kHz
            TA0CTL    |= ID_3 | TACLR;
            TA0EX0     = TAIDEX_7;                                   // 2.5 kHz
            break;
        case 1:
            TB0CTL    |= ID_3 | TBCLR;
            TB0EX0     = TBIDEX__5;                                  // 5 kHz
            TA0CTL    |= ID_3 | TACLR;
            TA0EX0     = TAIDEX_4;                                   // 5 kHz
            break;
        case 2:
            TB0CTL    |= ID_3 | TBCLR;
            TB0EX0     = TBIDEX__4;                                  // 6.25 kHz
            TA0CTL    |= ID_3 | TACLR;
            TA0EX0     = TAIDEX_3;                                   // 6.25 kHz
            break;
        case 3:
            TB0CTL    |= ID_3 | TBCLR;
            TB0EX0     = TBIDEX__2;                                  // 12.5 kHz
            TA0CTL    |= ID_3 | TACLR;
            TA0EX0     = TAIDEX_1;                                   // 12.5 kHz
            break;
        case 4:
            TB0CTL    &= ~ID_3;
            TB0EX0     = TBIDEX__1;
            TB0CTL    |=  PWM_FREQ_25KHZ | TBCLR;                    // 25 kHz
            TA0CTL    &= ~ID_3;
            TA0EX0     = TAIDEX_0;
            TA0CTL    |= PWM_FREQ_25KHZ | TACLR;                     // 25 kHz
            break;
        case 5:
            TB0CTL    &= ~ID_3;
            TB0EX0     = TBIDEX__1;
            TB0CTL    |=  PWM_FREQ_50KHZ  | TBCLR;                   // 50 kHz
            TA0CTL    &= ~ID_3;
            TA0EX0     = TAIDEX_0;
            TA0CTL    |= PWM_FREQ_50KHZ | TACLR;                     // 50 kHz
            break;
        case 6:
            TB0CTL    &= ~ID_3;
            TB0EX0     = TBIDEX__1;
            TB0CTL    |= PWM_FREQ_100KHZ  | TBCLR;                  // 100 kHz
            TA0CTL    &= ~ID_3;
            TA0EX0     = TAIDEX_0;
            TA0CTL    |= PWM_FREQ_100KHZ | TACLR;                   // 100 kHz
            break;
        default:
            break;
        }
        prev_pwmFreq = gBrushed_Obj.pwmFreq;
    }
}

// Function to control the duty cycle for PH_EN mode DRV8845
void drv_PH_EN_RampControl(void)
{
    set_TP1_debug_timing_hi;
    //Bridge-1 Control
    if (gBrushed_Obj.dir1  && !gBrushed_Obj.pre_dir1)                // Direction changed to Forward
    {
        drv_RampCTRL(&gBrushed_Obj.rampInput1, 0);                   // rampInput1 ramp down before direction reversal
        TA0CCR2 = gBrushed_Obj.rampInput1.setpointValue;
        if (!gBrushed_Obj.rampInput1.setpointValue)
        {
            gBrushed_Obj.pre_dir1 = 1;
            set_phase1_hi;                                           // Set direction to Forward
        }
    }
    else if (!gBrushed_Obj.dir1  && gBrushed_Obj.pre_dir1)           // Direction changed to Reverse
    {
        drv_RampCTRL(&gBrushed_Obj.rampInput1, 0);                   // in1 ramp down before direction reversal
        TA0CCR2 = gBrushed_Obj.rampInput1.setpointValue;             // Update Timer PWM duty cycle
        if (!gBrushed_Obj.rampInput1.setpointValue)
        {
            gBrushed_Obj.pre_dir1 = 0;
            set_phase1_lo;                                           // Set direction to Reverse
        }
    }
    else // in1 ramp for PH/EN during normal operation
    {
        P1SEL |= I01;                                                // Set I01/I11 to timer PWM output
        drv_RampCTRL(&gBrushed_Obj.rampInput1, gBrushed_Obj.in1);    // PWM ramp up or down
        TA0CCR2 = gBrushed_Obj.rampInput1.setpointValue;             // Update Timer PWM duty cycle
    } // End Bridge-1 Control

    //Bridge-2 Control
    if (gBrushed_Obj.dir2  && !gBrushed_Obj.pre_dir2)                // Direction changed to Forward
    {
        drv_RampCTRL(&gBrushed_Obj.rampInput2, 0);                   // in1 ramp down before direction reversal
        TA0CCR4 = gBrushed_Obj.rampInput2.setpointValue;
        if (!gBrushed_Obj.rampInput2.setpointValue)
        {
            gBrushed_Obj.pre_dir2 = 1;                              
            set_phase2_hi;                                           // Forward
        }
    }
    else if (!gBrushed_Obj.dir2  && gBrushed_Obj.pre_dir2)           // Direction changed to Reverse
    {
        drv_RampCTRL(&gBrushed_Obj.rampInput2, 0);                   // in2 ramp down before direction reversal
        TA0CCR4 = gBrushed_Obj.rampInput2.setpointValue;             // Update Timer PWM duty cycle
        if (!gBrushed_Obj.rampInput2.setpointValue)
        {
            gBrushed_Obj.pre_dir2 = 0;
            set_phase2_lo;                                           // Reverse
        }
    }
    else // in2 ramp for PH/EN during normal operation
    {
        P1SEL |= I02;                                                // Set IN2 to timer PWM output
        drv_RampCTRL(&gBrushed_Obj.rampInput2, gBrushed_Obj.in2);    // PWM ramp up or down
        TA0CCR4 = gBrushed_Obj.rampInput2.setpointValue;             // Update Timer PWM duty cycle
    } // End Bridge-2 Control

    //Bridge-3 Control
    if (gBrushed_Obj.dir3  && !gBrushed_Obj.pre_dir3)                // Direction changed to Forward
    {
        drv_RampCTRL(&gBrushed_Obj.rampInput3, 0);                   // rampInput3 ramp down before direction reversal
        TB0CCR2 = gBrushed_Obj.rampInput3.setpointValue;
        if (!gBrushed_Obj.rampInput3.setpointValue)
        {
            gBrushed_Obj.pre_dir3 = 1;
            set_phase3_hi;                                           // Set direction to Forward
        }
    }
    else if (!gBrushed_Obj.dir3  && gBrushed_Obj.pre_dir3)           // Direction changed to Reverse
    {
        drv_RampCTRL(&gBrushed_Obj.rampInput3, 0);                   // rampInput3 ramp down before direction reversal
        TB0CCR2 = gBrushed_Obj.rampInput3.setpointValue;             // Update Timer PWM duty cycle
        if (!gBrushed_Obj.rampInput3.setpointValue)
        {
            gBrushed_Obj.pre_dir3 = 0;
            set_phase3_lo;                                           // Set direction to Reverse
        }
    }
    else // rampInput3 ramp for PH/EN during normal operation
    {
        P4SEL |= I03;                                                // Set I03/I13 to timer PWM output
        drv_RampCTRL(&gBrushed_Obj.rampInput3, gBrushed_Obj.in3);    // PWM ramp up or down
        TB0CCR2 = gBrushed_Obj.rampInput3.setpointValue;             // Update Timer PWM duty cycle
    } // End Bridge-3 Control

    //Bridge-4 Control
    if (gBrushed_Obj.dir4  && !gBrushed_Obj.pre_dir4)                // Direction changed to Forward
    {
        drv_RampCTRL(&gBrushed_Obj.rampInput4, 0);                   // rampInput4 ramp down before direction reversal
        TB0CCR4 = gBrushed_Obj.rampInput4.setpointValue;
        if (!gBrushed_Obj.rampInput4.setpointValue)
        {
            gBrushed_Obj.pre_dir4 = 1;
            set_phase4_hi;                                           // Set direction to Forward
        }
    }
    else if (!gBrushed_Obj.dir4  && gBrushed_Obj.pre_dir4)           // Direction changed to Reverse
    {
        drv_RampCTRL(&gBrushed_Obj.rampInput4, 0);                   // rampInput4 ramp down before direction reversal
        TB0CCR4 = gBrushed_Obj.rampInput4.setpointValue;             // Update Timer PWM duty cycle
        if (!gBrushed_Obj.rampInput4.setpointValue)
        {
            gBrushed_Obj.pre_dir4 = 0;
            set_phase4_lo;                                           // Set direction to Reverse
        }
    }
    else // rampInput4 ramp for PH/EN during normal operation
    {
        P4SEL |= I04;                                                // Set I04/I14 to timer PWM output
        drv_RampCTRL(&gBrushed_Obj.rampInput4, gBrushed_Obj.in4);    // PWM ramp up or down
        TB0CCR4 = gBrushed_Obj.rampInput4.setpointValue;             // Update Timer PWM duty cycle
    } // End Bridge-4 Control
    
    set_TP1_debug_timing_lo;
} // End function to control the duty cycle for PH_EN mode DRV8845 Brushed Motors

// Function to ramp up or down the PWM duty cycle for in1, in2, in3 and in4
void drv_RampCTRL(volatile RMPCNTL_t* rmpCntl, uint8_t targetValue)
{
    if (rmpCntl->rampDelayMax == 0)
    {
        rmpCntl->setpointValue = targetValue;
    }
    else
    {
        if (rmpCntl->rampDelayCount++ >= rmpCntl->rampDelayMax)
        {
            if (targetValue > rmpCntl->setpointValue)
            {
                rmpCntl->setpointValue++;
            }
            else if (targetValue < rmpCntl->setpointValue)
            {
                rmpCntl->setpointValue--;
            }

            //TA0CCTL0 &= ~CCIE;                                       // Disable Timer A0 CCR0 interrupt
            rmpCntl->rampDelayCount = 0;
        }
    }
} // End function to ramp up and down the PWM duty cycle to target duty cycle value
