/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/********************************************************************************
 *
 *  utility.c - Definition file for utility functions common between 8845 and 8849
 *  DRV884x_EVM_FIRMWARE
 *  01/03/2023
 *  Jacob Thompson
 *  03/01/2023
 *  Murugavel Raju
 *
 *******************************************************************************/
           

/********************************************************************************
 * HEADER FILES
 *******************************************************************************/
#include "drv884x/include/drv8845.h"
#include "drv884x/include/utility.h"
#include "stepper.h"
#include "stepper_8845.h"
#include "uart.h"

/********************************************************************************
 * LOCAL ROUTINES
 *******************************************************************************/
// Clock Initialization
static inline void clock_Init(void)
{
    PMMCTL0 = PMMPW | PMMCOREV_3;             // PMM Core Voltage 1.85 V for 20 MHz CPU operation
    __delay_cycles(100000);                   // Settling delay for the core regulator

    UCSCTL3 = SELREF_2;                       // Set DCO FLL reference = REFO
    UCSCTL4 |= SELA_2 + SELS_3;               // Set ACLK = REFO and SMCLK = DCO
    UCSCTL1 = DCORSEL_6;                      // Select DCO range 20MHz operation
    UCSCTL2 = 609;                            // FLL multiplier
    __delay_cycles(625000);                   // Settling delay for DCO, 32 x 32 x 20 MHz / 32768 Hz = 625000 = MCLK cycles
}

// GPIO Initialization. This block configures all the DRV884x PINs.
static inline void gpio_Init(void)
{
    // Port P1 configuration
    P1OUT = 0;                                                       // All outputs low
    P1SEL = 0;                                                       // Clear P1SEL
    P1DIR = BIT7 | BIT6 | I02 | PHASE2 | I01 | PHASE1 | BIT1 | BIT0; // All are outputs
    
    // Port P2 configuration
    P2OUT = 0;                                                       // All outputs low
    P2SEL = 0;                                                       // Clear P2SEL
    P2DIR = DECAY1 | MODE11_PD | TP3 | TP2 | TP1;                    // All outputs except nFAULT1, nFAULT2

    // Port P3 configuration
    P3OUT = 0;                                                       // All outputs low
    P3SEL = 0;                                                       // Clear P3SEL
    P3DIR = nSLEEP | ENABLE1 | MODE01 | ENABLE2_STATUS | ENABLE1_STATUS | GUI_STATUS | MCU_STATUS | TP6; // All outputs
    // P3SEL = ENABLE2_STATUS;                                       // ENABLE2_STATUS = SMCLK for probing

    // Port P4 configuration
    P4OUT = 0;                                                       // All outputs low
    P4SEL = 0;                                                       // Clear P4SEL
    P4DIR = DECAY2_PD | DECAY1_PD | MODE12_PD | I04 | PHASE4 | I03 | PHASE3 | BIT0; // All are outputs

    // Port P5 configuration
    P5OUT = 0;                                                       // All outputs low
    P5DIR = TP4 | TP5 | BIT2;                                        // IDR_1, IDR_2, IDR_3, VREF+ and VREF- are inputs
    P5SEL = BIT1 | BIT0;                                             // P5.0 = VeREF+ and P5.1 = VeREF-

    // Port P6 configuration
    P6OUT = 0;                                                       // All outputs low
    P6DIR = VREF2_Pin | VREF1_Pin | BIT5 | BIT4 | BIT3 | BIT2 | BIT1;// VSEN_VM is input, all others outputs or NC
    P6SEL = VSEN_VM;                                                 // Analog inputs

    // Port P7 configuration
    P7OUT = 0;                                                       // All outputs low
    P7DIR = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;   // All pins are NC

    // Port P8 configuration
    P8OUT = 0;                                                       // All outputs low
    P8DIR = BIT7 | BIT6 | BIT5 | BIT4 | MODE01_PD | MODE02_PD;       // UART pins stay inputs 

    // Port P9 configuration
    P9OUT = 0;                                                       // All outputs low
    P9DIR = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;   // All pins are NC
}

// Watchdog initialization
static inline void watchdog_Init(void)
{
    WDTCTL = WDT_ADLY_1_9;                                           // Configure time interval for 1.9 ms using fACLK
    SFRIE1 |= WDTIE;                                                 // Interval timer interrupt enabled
}

// Read EVMID information of the EVM hardware
void getEVMID(void)
{
    /* Get identify of the EVM connected
     *
     * DRV8845 and DRV8849 EVMs have on board ID resistors inputs on
     * the MCU GPIO. ID1 to ID3 inputs uniquely identify the EVM connected
     * as well as whether it has pre-production or production silicon.
     *
     *   IDx=1       IDx=0      Pin state
     *     H                    Pull-up installed
     *                  L       Pull-down installed
     *
     *   P5.5 = ID3
     *   P5.4 = ID2 (not used)
     *   P5.3 = ID1
     *
     *           EVM HW   Package  Interface     Motor   ID1 ID2          ID3
     * DRV8845  MD061E1-001  VQFN   PHASE/INx    Stepper   H   L    0x2    L (pre-production)
     * DRV8849  MD061E1-002  VQFN   STEP/DIR     Stepper   L   L    0x1    L (pre-production)
     *
     * ID5 = L, preproduction silicon and ID5 = H, production silicon
     */

    // ID1
    if (read_ID1_pin)
        gDRV884x_Obj.deviceID = 8849;
    else
        gDRV884x_Obj.deviceID = 8845;

    // ID3
    if (read_ID3_pin)                                      // ID3 pull-up = production
        gDRV884x_Obj.revID = REV_PG2_0;
    else                                                   // ID3 pull-down = preproduction
        gDRV884x_Obj.revID = REV_PG1_0;
}

/*******************************************************************************
 * API IMPLEMENTATION
 *******************************************************************************/
// Initialize common GUI variables and stepper drivers GUI variables
void drv884x_init()
{
    gDRV884x_Obj.Firmware_Version = FIRMWARE_VERSION;
    gDRV884x_Obj.revID = 0;
    gDRV884x_Obj.nSleep = 0;
    gDRV884x_Obj.gDRVEN1 = 0;
    gDRV884x_Obj.gDRVEN2 = 0;
    gDRV884x_Obj.ClrFlt = 0;
    gDRV884x_Obj.MotorMode = 0;
    gDRV884x_Obj.MotorModePrevious = 0;
    gDRV884x_Obj.DecayMode1 = DEFAULT_DecayMode;
    gDRV884x_Obj.DecayMode2 = DEFAULT_DecayMode;
    MotorModePrevious2 = 0;
}

// Initialize GUI default values
void SetDefaultGUIVals()
{
    MOTOR_CMD_ACTIVE1_STEPPER1      = false;
    MOTOR_CMD_ACTIVE2_STEPPER1      = false;
    MOTOR_CMD_ACTIVE3_STEPPER1      = false;
    gStepper_Obj1.STARTING_SPEED    = DEFAULT_START_SPEED;     // initial speed in pulses per second when starting motor
    gStepper_Obj1.TARGET_SPEED      = DEFAULT_TARGET_SPEED;    // target speed of motor
    gStepper_Obj1.ACCEL_RATE        = DEFAULT_ACCEL_RATE;      // acceleration rate from starting speed to target speed
    gStepper_Obj1.STOPPING_SPEED    = DEFAULT_STOPPING_SPEED;  // speed to stop motor once deceleration has reached this point
    gStepper_Obj1.NUM_STEPS         = DEFAULT_NUM_STEPS;       // number of steps to move when manually stepping
    gStepper_Obj1.STEPS_2_STOP      = DEFAULT_STEPS_2_STOP;    // Number of steps used to stop the motor when manually stepping
    gStepper_Obj1.RUN_STEPPER       = false;                   // Auto run enabled
    gStepper_Obj1.MANUAL_ADVANCE    = false;                   // Manually advance motor
    gStepper_Obj1.RECIPROCATION     = false;
    gStepper_Obj1.gDIR              = false;
    gStepper_Obj1.IFS               = DEFAULT_IFS;             // Default current setting for DRV8849

    if (gDRV884x_Obj.deviceID == 8845)
    {
        gBrushed_Obj.VREF1          = DEFAULT_CURRENT_SETTING; // Default current setting for DRV8845
        gDRV884x_Obj.StepMode1      = DEFAULT_STEP_MODE_8845;  // default microstepping mode 1/16 uSteps
    }
    else if (gDRV884x_Obj.deviceID == 8849)
    {
        gBrushed_Obj.VREF1          = DEFAULT_FSCURRENT_8849;  // Default current setting for DRV8849
        gDRV884x_Obj.StepMode1      = DEFAULT_STEP_MODE_8849;  // Default microstepping mode 1/16 uSteps
    }


    MOTOR_CMD_ACTIVE1_STEPPER2      = false;
    MOTOR_CMD_ACTIVE2_STEPPER2      = false;
    MOTOR_CMD_ACTIVE3_STEPPER2      = false;
    gStepper_Obj2.STARTING_SPEED    = DEFAULT_START_SPEED;     // initial speed in pulses per second when starting motor
    gStepper_Obj2.TARGET_SPEED      = DEFAULT_TARGET_SPEED;    // speed of motor once running
    gStepper_Obj2.ACCEL_RATE        = DEFAULT_ACCEL_RATE;      // acceleration rate from starting speed to target speed
    gStepper_Obj2.STOPPING_SPEED    = DEFAULT_STOPPING_SPEED;  // speed to stop motor once deceleration has reached this point
    gStepper_Obj2.NUM_STEPS         = DEFAULT_NUM_STEPS;       // number of steps to move when manually stepping
    gStepper_Obj2.STEPS_2_STOP      = DEFAULT_STEPS_2_STOP;    // number of steps used to stop the motor when manually stepping
    gStepper_Obj2.RUN_STEPPER       = false;                   // Auto run enabled
    gStepper_Obj2.MANUAL_ADVANCE    = false;                   // Manually advance motor
    gStepper_Obj2.RECIPROCATION     = false;
    gStepper_Obj2.gDIR              = false;
    gStepper_Obj2.IFS               = DEFAULT_IFS;             // Default current setting for DRV8849

    if (gDRV884x_Obj.deviceID == 8845)
    {
        gBrushed_Obj.VREF2          = DEFAULT_CURRENT_SETTING; // Default current setting for DRV8845
        gDRV884x_Obj.StepMode2      = DEFAULT_STEP_MODE_8845;  // default microstepping mode 1/16 uSteps
    }
    else if (gDRV884x_Obj.deviceID == 8849)
    {
        gBrushed_Obj.VREF2          = DEFAULT_FSCURRENT_8849;  // Default current setting for DRV8849
        gDRV884x_Obj.StepMode2      = DEFAULT_STEP_MODE_8849;  // Default microstepping mode 1/16 uSteps
    }

    // set boolean defaults
    LastCommandWasRunSteps1         = false;
    MoveSteps1                      = false;
    UpdateSteppingRateTMR1          = false;
    UseSlowTimerClk1                = false;
    ok_to_start_timer1              = true;
    tmpReciprocation1               = false;
    last_reciprocation_state1       = false;
    UpdateMotorParameters1          = true;

    LastCommandWasRunSteps2         = false;
    MoveSteps2                      = false;
    UpdateSteppingRateTMR2          = false;
    UseSlowTimerClk2                = false;
    ok_to_start_timer2              = true;
    tmpReciprocation2               = false;
    last_reciprocation_state2       = false;
    UpdateMotorParameters2          = true;
}

// Clear Fault function for Hardware configuration
void ClearFaultPulse()
{
    // Clear fault with nFAULT reset pulse
    clear_fault_loop = 0;
    while (clear_fault_loop < 1)
    {
        clear_fault_loop ++;
        set_nSLEEP_lo;                                     // pulse low nSLEEP
        _delay_cycles(600);                                // ~30us @ 20 MHz MCLK
        set_nSLEEP_hi;                                     // nSLEEP back to high
    }
}

// MSP430F5338 MCU initialization
void mcu_init()
{
    WDTCTL = WDTPW | WDTHOLD;                              // Stop Watchdog Timer on POR and PUC
    clock_Init();
    uartInit();
    gpio_Init();
    watchdog_Init();
} // End MSP430F5338 MCU initialization
