/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *  main.c - c file for main function
 *  DRV884x_EVM_FIRMWARE
 *  08/25/2022
 *  Jacob Thompson
 *  03/01/2023
 *  Murugavel Raju
 ******************************************************************************/
#include <msp430.h>
#include "drv884x/include/drv8845.h"
#include "drv884x/include/utility.h"
#include "stepper.h"
#include "stepper_8845.h"
#include "Serial_Cmd_Monitor.h"


// Main
int main(void)
{
    interval_tick = 0;
    interval_count = 4;                                    // 8ms wake up interval

    mcu_init();                                            // Initialize MCU peripherals
    drv884x_init();                                        // Device ID, firmware version and GUI stepper variables configurations
    getEVMID();                                            // Read EVM ID and set value for deviceID global variable

    // Initialize for Stepper Motor control
    if (gDRV884x_Obj.deviceID == 8845)
        stepper_8845_Init();
    else if(gDRV884x_Obj.deviceID == 8849)
        stepper_8849_Init();

    // Serial protocol between GUI USB and MCU UART initialization
    ClearBufferRelatedParam();                             // GUI Composer Monitor Initialization
    __bis_SR_register(GIE);                                // Global interrupts enable

    while (1)
    {
        // set_TP_debug_timing_hi;                            // For debug and timing measurements

        // Change GPIO configuration or timers if DRV8845 changed MotorModes
        if (gDRV884x_Obj.deviceID == 8845 && (gDRV884x_Obj.MotorMode != gDRV884x_Obj.MotorModePrevious))
        {
            // Stepper Motor routine initialization
            if (gDRV884x_Obj.MotorMode == 0)
            {
                stepper_8845_Init();                       // Initialize for stepper mode
            }
            // Brushed DC Motor initialization
            else if (gDRV884x_Obj.MotorMode == 1)
            {
                brushed_init();                            // GUI brushed variables configurations
                brushedTimerConfig();                      // Timer PWM configuration and initialization
            }

            gDRV884x_Obj.MotorModePrevious = gDRV884x_Obj.MotorMode; // Reset previous now that GPIO setup correct
        }

        // Stepper motor driver variants or mode
        if (gDRV884x_Obj.deviceID == 8845 && gDRV884x_Obj.MotorMode == 0) // DRV8845 stepper mode
        {
            UpdateDeviceControls_Stepper_DRV8845();        // read the GUI values and pass to device
            UpdateGUIControls_DRV8845();                   // read device values and pass to GUI
            UpdateSpeed_8845();                            // control the motor based on info captured above
        }
        else if (gDRV884x_Obj.deviceID == 8849)            // DRV8849 stepper
        {
            UpdateDeviceControls_Stepper_DRV8849();        // read the GUI values and pass to device
            UpdateGUIControls_DRV8849();                   // read device values and pass to GUI
            UpdateSpeed_8849();                            // control the motor based on info captured above
            SetMotorParameters_8849();                     // Parse stepper motor control parameters from GUI inputs
            DetermineNextAction_8849();                    // Parse stepper motion profile
        }
        else if (gDRV884x_Obj.deviceID == 8845 && gDRV884x_Obj.MotorMode == 1)
        {                                                  // DRV8845 Brushed DC motor driver mode
            UpdateDeviceControls_Brushed_DRV8845();        // read the GUI values and update device control values
            drv_PH_EN_RampControl();
        }// End Stepper motor driver variants or mode

        // set_TP_debug_timing_lo;                            // For debug and timing measurements
        __bis_SR_register(LPM0_bits + GIE);                // Enter LPM0 Low Power Mode 0
    }
}

// Periodic wake-up and Status LED derived from the watchdog interval timer ticks
#pragma vector=WDT_VECTOR
__interrupt void WatchDog_Timer(void)
{
    if (interval_tick++ == interval_count)                 // interval_count x 1.9 ms = ~ 8 ms wake-up interval
    {
        interval_tick = 0;                                 // Clear interval tick
        LED_TOGGLE_CNT++;
        __bic_SR_register_on_exit(LPM0_bits);              // Clear LPM0 bits (Low Power Mode 0)
    }
}
