/* --COPYRIGHT--,BSD
 * Copyright (c) 2016,2019, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/********************************************************************************
 *
 *  stepper.c - Definition file for functions related to stepper motion control
 *  DRV8849_Example FIRMWARE
 *  10/28/2020
 *  updated 01/20/2022
 *  update 03/01/2023
 *  Murugavel Raju
 *
 *******************************************************************************/

/********************************************************************************
 * HEADER FILES
 *******************************************************************************/
#include "stepper.h"

/********************************************************************************
 * MACROS
 *******************************************************************************/
#pragma DATA_SECTION(gStepper_Obj1,".stp1VarSection");
#pragma DATA_SECTION(gStepper_Obj2,".stp2VarSection");
volatile Stepper_Driving_Obj_t gStepper_Obj1;       // Stepper 1
volatile Stepper_Driving_Obj_t gStepper_Obj2;       // Stepper 2

/********************************************************************************
 * LOCAL ROUTINES
 *******************************************************************************/
// DRV8849 (stepper only) initialization for stepper motor
// TimerB and TimerD Initialization for PWM
void stepper_8849_Init(void)
{
    // DAC12 configuration
    DAC12_0CTL0 =  DAC12CALON | DAC12SREF_3 | DAC12IR | DAC12AMP_7;  // Configure with eVREF+ DAC12_0 on P6.6 initiate DAC12_1 calibration
    while (DAC12_0CTL0 & DAC12CALON);                                // Wait for Calibration complete
    DAC12_0DAT = 0;                                                  // Default output 0V

    DAC12_1CTL0 =  DAC12CALON | DAC12SREF_3 | DAC12IR | DAC12AMP_7;  // Configure with eVREF+ DAC12_1 on P6.7 initiate DAC12_1 calibration
    while (DAC12_1CTL0 & DAC12CALON);                                // Wait for Calibration complete
    DAC12_1DAT = 0;                                                  // Default output 0V
    //End DAC12 configuration

    enable_Stepper1Timer_Interrupt = true;                           // Enable Timer Interrupt
    enable_Stepper2Timer_Interrupt = true;                           // Enable Timer Interrupt
    set_ENABLE1_lo;
    set_ENABLE2_lo;
    set_DIR1_hi;
    set_DIR2_hi;                                                     //to match the GUI

    // configure Timer B0 for STEP2 output
    TB0CTL = TBSSEL_2 | ID_1 | MC_0 | TBCLR;                         // use 10 MHz clock, SMCLK / 2
    P4SEL |= STEP2;                                                  // STEP1 pin GPIO select as PWM output for Timer B0
    // configure Timer A1 for acceleration and deceleration control for Stepper 2
    TA1EX0 = TAIDEX_7;                                               // select TACLK = SMCLK/8
    TA1CTL = TASSEL_2 | ID_3 | MC_0 | TACLR;                         // select TACLK, divide by 8, stop

    // configure Timer A0 for STEP1 output
    TA0CTL = TASSEL_2 | ID_1 | MC_0 | TACLR;                         // use 10 MHz clock, SMCLK / 2
    P1SEL |= STEP1;                                                  // STEP1 pin GPIO select
    // configure Timer A2 for acceleration and deceleration control for Stepper 1
    TA2EX0 = TAIDEX_7;                                               // select TACLK = SMCLK/8
    TA2CTL = TASSEL_2 | ID_3 | MC_0 | TACLR;                         // select TACLK, divide by 8, stop

    // Set Default GUI Variables Values
    SetDefaultGUIVals();
}

// Routine to communicate between GUI and device
// reads GUI variables and updates the GPIOs
void UpdateDeviceControls_Stepper_DRV8849()
{
    // toggle MCU_STATUS LED if count is reached
    if (LED_TOGGLE_CNT > LED_TIMEOUT)
    {
        toggle_MCUstatus_led;
        LED_TOGGLE_CNT = 0;

        // Measure VM and update to GUI display
        ADC12CTL0 = ADC12ON;                                         // Enable ADC
        ADC12IE = 0;                                                 // Disable ADC interrupts
        ADC12CTL1 = ADC12SHP;                                        // configure for single conversion
        ADC12MCTL0 = ADC12INCH_0 + ADC12SREF_6;                      // A0 for VM measurement
        ADC12CTL0 |= ADC12SC + ADC12ENC;                             // Enable and start conversion
        while (ADC12CTL1 & ADC12BUSY);                               // Wait for conversion to complete
        gDRV884x_Obj.VMvoltage = ADC12MEM0;                          // Conversion result stored in ADC12MEM0 sent to GUI display
        if (gDRV884x_Obj.VMvoltage < ADC12scale)
            gDRV884x_Obj.VMvoltage = 0;                              // VM is 0
        ADC12CTL0 &= ~ADC12ENC;                                      // Disable ADC
        // End measure VM and update to GUI display
    }

    // From GUI to MCU
    // Update the GPIO Pins from Global Variables

    // Decay Mode update for Stepper 1
    switch (gDRV884x_Obj.DecayMode1)
    {
    case SMART_TUNE:                                                 // Slow Decay
        set_DECAY1_SmartTune;
        break;
    case STDD_7us:                                                   // Increasing steps Slow; Decreasing steps Mixed decay30% fast
        set_DECAY1_DD7;
        break;
    case STDD_16us:                                                  // Increasing steps Slow; Decreasing steps Mixed decay60% fast
        set_DECAY1_DD16;
        break;
    case STDD_32us:                                                  // Increasing steps Slow; Decreasing steps Fast
        set_DECAY1_DD32;
        break;
    }

    // Decay Mode update for Stepper 2
    switch (gDRV884x_Obj.DecayMode2)
    {
    case SMART_TUNE:                                                 // Slow Decay
        set_DECAY2_SmartTune;
        break;
    case STDD_7us:                                                   // Increasing steps Slow; Decreasing steps Mixed decay30% fast
        set_DECAY2_DD7;
        break;
    case STDD_16us:                                                  // Increasing steps Slow; Decreasing steps Mixed decay60% fast
        set_DECAY2_DD16;
        break;
    case STDD_32us:                                                  // Increasing steps Slow; Decreasing steps Fast
        set_DECAY2_DD32;
        break;
    }

    // Step Mode settings update for Stepper 1
    switch (gDRV884x_Obj.StepMode1)
    {
    case FULLSTEP_100:                                               // Full step (2-phase excitation) with 100% current
        set_fullstep_100_Stepper1;
        break;
    case FULLSTEP_71:                                                // Full step (2-phase excitation) with 71% current
        set_fullstep_71_Stepper1;
        break;
    case NONCIRCLR_uSTEP_2:                                          // Non-circular 1/2 step
        set_noncircular_halfstep_Stepper1;
        break;
    case uSTEP_2:                                                    // half step
        set_halfstep_Stepper1;
        break;
    case uSTEP_4:                                                    // 1/4 step
        set_microstep_4_Stepper1;
        break;
    case uSTEP_8:                                                    // 1/8 step
        set_microstep_8_Stepper1;
        break;
    case uSTEP_16:                                                   // 1/16 step
        set_microstep_16_Stepper1;
        break;
    case uSTEP_32:                                                   // 1/32 step
        set_microstep_32_Stepper1;
        break;
    case uSTEP_64:                                                   // 1/64 step
        set_microstep_64_Stepper1;
        break;
    case uSTEP_128:                                                  // 1/128 step
        set_microstep_128_Stepper1;
        break;
    case uSTEP_256:                                                  // 1/256 step
        set_microstep_256_Stepper1;
        break;
    }

    // Step Mode settings update for Stepper 2
    switch (gDRV884x_Obj.StepMode2)
    {
    case FULLSTEP_100:                                               // Full step (2-phase excitation) with 100% current
        set_fullstep_100_Stepper2;
        break;
    case FULLSTEP_71:                                                // Full step (2-phase excitation) with 71% current
        set_fullstep_71_Stepper2;
        break;
    case NONCIRCLR_uSTEP_2:                                          // Non-circular 1/2 step
        set_noncircular_halfstep_Stepper2;
        break;
    case uSTEP_2:                                                    // half step
        set_halfstep_Stepper2;
        break;
    case uSTEP_4:                                                    // 1/4 step
        set_microstep_4_Stepper2;
        break;
    case uSTEP_8:                                                    // 1/8 step
        set_microstep_8_Stepper2;
        break;
    case uSTEP_16:                                                   // 1/16 step
        set_microstep_16_Stepper2;
        break;
    case uSTEP_32:                                                   // 1/32 step
        set_microstep_32_Stepper2;
        break;
    case uSTEP_64:                                                   // 1/64 step
        set_microstep_64_Stepper2;
        break;
    case uSTEP_128:                                                  // 1/128 step
        set_microstep_128_Stepper2;
        break;
    case uSTEP_256:                                                  // 1/256 step
        set_microstep_256_Stepper2;
        break;
    }

    // Fault Status monitoring
    // nFAULT 1 (for Stepper 1)
    if (!(read_nFAULT1_pin) && gDRV884x_Obj.nFault1)                 // Fault Condition
    {
        gDRV884x_Obj.nFault1 = 0;
    }
    else if (read_nFAULT1_pin)
        gDRV884x_Obj.nFault1 = 1;

    // nFAULT 2 (for Stepper 2)
    if (!(read_nFAULT2_pin) && gDRV884x_Obj.nFault2)                 // Fault Condition
    {
        gDRV884x_Obj.nFault2 = 0;
    }
    else if (read_nFAULT2_pin)
        gDRV884x_Obj.nFault2 = 1;

    // nSLEEP control
    if (gDRV884x_Obj.nSleep && !(read_nSLEEP_pin))
    {
        set_nSLEEP_hi;
        _delay_cycles(30000);                                        // 1.5 ms wake up delay
    }
    else if (!gDRV884x_Obj.nSleep && (read_nSLEEP_pin))
    {
        set_nSLEEP_lo;
        set_ENABLE1_led_lo;
        set_ENABLE2_led_lo;
    }

    // DIR for Stepper 1
    if (gStepper_Obj1.gDIR  && nextDIR1)
    {
        if (!(gStepper_Obj1.MANUAL_ADVANCE) && gStepper_Obj1.RUN_STEPPER) 
        {
            ok_to_start_timer1 = false;
            gStepper_Obj1.RUN_STEPPER = false;
            MOTOR_CMD_ACTIVE1_STEPPER1 = false;
            MOTOR_CMD_ACTIVE2_STEPPER1 = false;
            MOTOR_CMD_ACTIVE3_STEPPER1 = false;
            __delay_cycles(20000);                                   // 1ms delay
            set_DIR1_hi;
            nextDIR1 = 0;
            gStepper_Obj1.RUN_STEPPER = true;
            ok_to_start_timer1 = true;
        }
        else
        {
            set_DIR1_hi;
            nextDIR1 = 0;
        }
    }
    else if (!(gStepper_Obj1.gDIR)  && !nextDIR1)
    {
        if (!(gStepper_Obj1.MANUAL_ADVANCE) && gStepper_Obj1.RUN_STEPPER)
        {
            ok_to_start_timer1 = false;
            gStepper_Obj1.RUN_STEPPER  = false;
            MOTOR_CMD_ACTIVE1_STEPPER1 = false;
            MOTOR_CMD_ACTIVE2_STEPPER1 = false;
            MOTOR_CMD_ACTIVE3_STEPPER1 = false;
            __delay_cycles(20000);                                   // 1ms delay
            set_DIR1_lo;
            nextDIR1 = 1;
            gStepper_Obj1.RUN_STEPPER = true;
            ok_to_start_timer1 = true;
        }
        else
        {
            set_DIR1_lo;
            nextDIR1 = 1;
        }
    }

    // DIR for Stepper 2
     if (gStepper_Obj2.gDIR  && nextDIR2)
     {
         if (!(gStepper_Obj2.MANUAL_ADVANCE) && gStepper_Obj2.RUN_STEPPER)
         {
             ok_to_start_timer2 = false;
             gStepper_Obj2.RUN_STEPPER = false;
             MOTOR_CMD_ACTIVE1_STEPPER2 = false;
             MOTOR_CMD_ACTIVE2_STEPPER2 = false;
             MOTOR_CMD_ACTIVE3_STEPPER2 = false;
             __delay_cycles(20000);                                  // 1ms delay
             set_DIR2_hi;
             nextDIR2 = 0;
             gStepper_Obj2.RUN_STEPPER = true;
             ok_to_start_timer2 = true;
         }
         else
         {
             set_DIR2_hi;
             nextDIR2 = 0;
         }
     }
     else if (!(gStepper_Obj2.gDIR)  && !nextDIR2)
     {
         if (!(gStepper_Obj2.MANUAL_ADVANCE) && gStepper_Obj2.RUN_STEPPER)
         {
             ok_to_start_timer2 = false;
             gStepper_Obj2.RUN_STEPPER = false;
             MOTOR_CMD_ACTIVE1_STEPPER2 = false;
             MOTOR_CMD_ACTIVE2_STEPPER2 = false;
             MOTOR_CMD_ACTIVE3_STEPPER2 = false;
             __delay_cycles(20000);                                  // 1ms delay
             set_DIR2_lo;
             nextDIR2 = 1;
             gStepper_Obj2.RUN_STEPPER = true;
             ok_to_start_timer2 = true;
         }
         else
         {
             set_DIR2_lo;
             nextDIR2 = 1;
         }
     } // End DIR for Stepper 2

    // Set ENABLE for Stepper 1 pin high to enable device based on widget setting
    if (gDRV884x_Obj.gDRVEN1)
    {
        if (!(read_ENABLE1_pin))
        {
            set_ENABLE1_hi;
            set_ENABLE1_led_hi;
        }
    }
    else
    {
        if (read_ENABLE1_pin)
        {
            set_ENABLE1_lo;
            set_ENABLE1_led_lo;
        }
    }

    // Set ENABLE for Stepper 2 pin high to enable device based on widget setting
    if (gDRV884x_Obj.gDRVEN2)
    {
        if (!(read_ENABLE2_pin))
        {
            set_ENABLE2_hi;
            set_ENABLE2_led_hi;
        }
    }
    else
    {
        if (read_ENABLE2_pin)
        {
            set_ENABLE2_lo;
            set_ENABLE2_led_lo;
        }
    }

    // Clear Fault
    if (gDRV884x_Obj.ClrFlt)
    {
        ClearFaultPulse();
        gDRV884x_Obj.ClrFlt = 0;
    }
} // End UpdateDeviceControls_Stepper_DRV8849


// Routine to communicate between device and GUI. Reads GPIOs and updates the GUI. Used for DRV8849 in Stepper mode
void UpdateGUIControls_DRV8849()
{
    // Disable Stepper Motor 1 control buttons
    if (gDRV884x_Obj.gDRVEN1 == 0 || gDRV884x_Obj.nSleep == 0)
    {
        // when disabled reset the indicators in the motor operation
        gStepper_Obj1.RUN_STEPPER       = false;
        gStepper_Obj1.MANUAL_ADVANCE    = false;
        gStepper_Obj1.RECIPROCATION     = false;
        MOTOR_CMD_ACTIVE1_STEPPER1      = false;
        MOTOR_CMD_ACTIVE2_STEPPER1      = false;
        MOTOR_CMD_ACTIVE3_STEPPER1      = false;

        // stop the timer
        TA0CTL &= ~MC_3;
        TA0CTL |= TACLR;
        
        // clear temp_reciprocation and MoveSteps so next movement is correct
        tmpReciprocation1 = false;                                   // reset reciprocation
        MoveSteps1 = false;                                          // clear MoveSteps1 so next movement is correct
        gStepper_Obj1.AccelerateState = IDLE;                        // set AccelerateState to idle when sleeping or disabled
    }

    // Disable Stepper Motor 2 control buttons
    if (gDRV884x_Obj.gDRVEN2 == 0 || gDRV884x_Obj.nSleep == 0)
    {
        // when disabled reset the indicators in the motor operation
        gStepper_Obj2.RUN_STEPPER = false;
        gStepper_Obj2.MANUAL_ADVANCE = false;
        MOTOR_CMD_ACTIVE1_STEPPER2 = false;
        MOTOR_CMD_ACTIVE2_STEPPER2 = false;
        MOTOR_CMD_ACTIVE3_STEPPER2 = false;

        // stop the timer
        TB0CTL &= ~MC_3;
        TB0CTL |= TBCLR;

        // clear temp_reciprocation and MoveSteps so next movement is correct
        tmpReciprocation2 = false;                                  // reset reciprocation
        MoveSteps2 = false;                                         // clear MoveSteps2 so next movement is correct
        gStepper_Obj2.AccelerateState = IDLE;                       // set AccelerateState to idle when sleeping or disabled
    }

    // Full Scale Current for Stepper 1
    if (gBrushed_Obj.VREF1 != IFS_previous1)
    {
        DAC12_0DAT = 2730*gBrushed_Obj.VREF1;
        IFS_previous1 = gBrushed_Obj.VREF1;
        _nop();
    }

    // Full Scale Current for Stepper 2
    if (gBrushed_Obj.VREF2 != IFS_previous2)
    {
        DAC12_1DAT = 2730*gBrushed_Obj.VREF2;
        IFS_previous2 = gBrushed_Obj.VREF2;
        _nop();
    }
} // End UpdateGUIControls_DRV884x()

// Routine to determine if any changes are in motor operation are required and pass the information to the motor state machine
void UpdateSpeed_8849()
{
    // Stepper 1 Speed
    StartingSpeed1 = (unsigned long) gStepper_Obj1.STARTING_SPEED << use_one_edge;       // Configure the starting speed

    // if StartingSpeed1 is < 160 change SMCLK_FREQ to 32768 Hz
    if ((gStepper_Obj1.STARTING_SPEED < 160) || (gStepper_Obj1.STOPPING_SPEED < 160))
        UseSlowTimerClk1 = true;                                                         // set Timer CLK to 32768 Hz and modify results using SMCLK_LOW_FREQ
    else
        UseSlowTimerClk1 = false;

    StoppingSpeed1      = (unsigned long) gStepper_Obj1.STOPPING_SPEED << use_one_edge;  // Configure the stopping speed
    StepsToMove1        = (unsigned long) gStepper_Obj1.NUM_STEPS;
    StepsToStop1        = (unsigned long) gStepper_Obj1.STEPS_2_STOP << use_one_edge;    // adjust for both edges
    DesiredTargetSpeed1 = (unsigned long) gStepper_Obj1.TARGET_SPEED << use_one_edge;    // Configure the target speed
    AccelRate1          = (unsigned long) gStepper_Obj1.ACCEL_RATE << use_one_edge;      // Configure the acceleration rate here
    
    // Stepper 2 Speed
    StartingSpeed2 = (unsigned long) gStepper_Obj2.STARTING_SPEED << use_one_edge;       // Configure the starting speed

    // if StartingSpeed1 is < 160 change SMCLK_FREQ to 32768 Hz
    if ((gStepper_Obj2.STARTING_SPEED < 160) || (gStepper_Obj2.STOPPING_SPEED < 160))
        UseSlowTimerClk2 = true;                                                         // set Timer CLK to 32768 Hz and modify results using SMCLK_LOW_FREQ
    else
        UseSlowTimerClk2 = false;
        
    StoppingSpeed2      = (unsigned long) gStepper_Obj2.STOPPING_SPEED << use_one_edge;  // Configure the stopping speed
    StepsToMove2        = (unsigned long) gStepper_Obj2.NUM_STEPS;
    StepsToStop2        = (unsigned long) gStepper_Obj2.STEPS_2_STOP << use_one_edge;    // adjust for both edges
    DesiredTargetSpeed2 = (unsigned long) gStepper_Obj2.TARGET_SPEED << use_one_edge;    // Configure the target speed
    AccelRate2          = (unsigned long) gStepper_Obj2.ACCEL_RATE << use_one_edge;      // Configure the acceleration rate here


    DetermineMotorMovement_8849();                                                       // Once type of motor movement is determined
    MotorStateMachine_8849();                                                            // Command the motor to move

    // enable or disable Timer A0 interrupt based on signal (Stepper 1)
    if (enable_Stepper1Timer_Interrupt && !(TA0CCTL0 & CCIE))
        TA0CCTL0 |= CCIE;
    else if (!enable_Stepper1Timer_Interrupt && (TA0CCTL0 & CCIE))
        TA0CCTL0 &= ~CCIE;

    // enable or disable Timer B0 interrupt based on signal (Stepper 2)
    if (enable_Stepper2Timer_Interrupt && !(TB0CCTL0 & CCIE))
        TB0CCTL0 |= CCIE;
    else if (!enable_Stepper2Timer_Interrupt && (TB0CCTL0 & CCIE))
        TB0CCTL0 &= ~CCIE;
} // End UpdateSpeed_8849

//  Routine to determine desired motor operation, direction, acceleration, deceleration, steps to move
void DetermineMotorMovement_8849()
{
    // Stepper 1 - Calculate steps to target and steps to stop prior to starting motor
    if ((gStepper_Obj1.RUN_STEPPER == true || gStepper_Obj1.MANUAL_ADVANCE == true || gStepper_Obj1.RECIPROCATION == true)
            && MOTOR_CMD_ACTIVE1_STEPPER1 == false
            && MOTOR_CMD_ACTIVE2_STEPPER1 == false
            && MOTOR_CMD_ACTIVE3_STEPPER1 == false)
    {
        // update steps to speed and steps to stop calculations
        // only calculate when the motor is idle and commanded to move
        time1 = (float) (DesiredTargetSpeed1 - StartingSpeed1) / AccelRate1;
        time_squared1 = time1 * time1;
        // calculate steps to target
        Steps2Target1 = ((AccelRate1 >> 1) * (time_squared1)) + (StartingSpeed1 * time1);
        // calculate Steps2Stop1 if:
        //  StepsToStop1 in GUI is >1 and StoppingSpeed1 is not equal Starting Speed
        if ((StepsToStop1 == 2 && use_one_edge == 1) || StepsToStop1 == 1)
        {
            if (StartingSpeed1 != StoppingSpeed1)
            {   // recalculate time1 if necessary
                time1 = (float) (DesiredTargetSpeed1 - StoppingSpeed1) / AccelRate1;
                time_squared1 = time1 * time1;
            }
            // calculate steps required to stop
            Steps2Stop1 = ((AccelRate1 >> 1) * (time_squared1)) + (StartingSpeed1 * time1);
        }
        else    
            Steps2Stop1 = StepsToStop1;                                                  // defined number of steps to stop

        // calculate stepping rate update (integer value divide by 32)
        if (AccelRate1 > 31 || AccelRate1 == 0)                                          // if >=32 or 0
            SteppingRateUpdate1 = AccelRate1 >> 5;                                       // divide by 32 by right shifting 5 positions
        else
            SteppingRateUpdate1 = 1;                                                     // set it to 1 to get correct value
        
        // Special case -- override prior settings and allow motor to attempt to run only at starting speed
        if (StartingSpeed1 >= DesiredTargetSpeed1)
        {
            SteppingRateUpdate1 = 0;
            Steps2Target1 = 0;
            Steps2Stop1 = 0;
        }

        // transition from ACCEL to DECEL when count equals (StepsToMove1 - Steps2Stop1) <=0
        if ((Steps2Stop1 > (StepsToMove1 >> 1)) && (StartingSpeed1 == StoppingSpeed1))
            Steps2Decel1 = StepsToMove1 >> 1;                                            // decel at midpoint
        else
            Steps2Decel1 = StepsToMove1 - Steps2Stop1;
    } // End Stepper 1 - Calculate steps to target and steps to stop prior to starting motor


    // Stepper 2 - Calculate steps to target and steps to stop prior to starting motor
    if ((gStepper_Obj2.RUN_STEPPER == true || gStepper_Obj2.MANUAL_ADVANCE == true || gStepper_Obj2.RECIPROCATION == true)
            && MOTOR_CMD_ACTIVE1_STEPPER2 == false
            && MOTOR_CMD_ACTIVE2_STEPPER2 == false
            && MOTOR_CMD_ACTIVE3_STEPPER2 == false)
    {
        // update steps to speed and steps to stop calculations
        // only calculate when the motor is idle and commanded to move
        time2 = (float) (DesiredTargetSpeed2 - StartingSpeed2) / AccelRate2;
        time_squared2 = time2 * time2;
        // calculate steps to target
        Steps2Target2 = ((AccelRate2 >> 1) * (time_squared2)) + (StartingSpeed2 * time2);
        // calculate Steps2Stop if:
        //  StepsToStop in GUI is >1 and StoppingSpeed is not equal Starting Speed
        if ((StepsToStop2 == 2 && use_one_edge == 1) || StepsToStop2 == 1)
        {
            if (StartingSpeed2 != StoppingSpeed2)
            {   // recalculate time2 if necessary
                time2 = (float) (DesiredTargetSpeed2 - StoppingSpeed2) / AccelRate2;
                time_squared2 = time2 * time2;
            }
            // calculate steps required to stop
            Steps2Stop2 = ((AccelRate2 >> 1) * (time_squared2)) + (StartingSpeed2 * time2);
        }
        else
            Steps2Stop2 = StepsToStop2;                 // defined number of steps to stop

        // calculate stepping rate update (integer value divide by 32)
        if (AccelRate2 > 31 || AccelRate2 == 0)         // if >=32 or 0
            SteppingRateUpdate2 = AccelRate2 >> 5;      // divide by 32 by right shifting 5 positions
        else
            SteppingRateUpdate2 = 1;                    // set it to 1 to get correct value

        // Special case -- override prior settings and allow motor to attempt to run only at starting speed
        if (StartingSpeed2 >= DesiredTargetSpeed2)
        {
            SteppingRateUpdate2 = 0;
            Steps2Target2 = 0;
            Steps2Stop2 = 0;
        }

        // transition from ACCEL to DECEL when count equals (StepsToMove - Steps2Stop) <=0
        if ((Steps2Stop2 > (StepsToMove2 >> 1)) && (StartingSpeed2 == StoppingSpeed2))
            Steps2Decel2 = StepsToMove2 >> 1;           // decel at midpoint
        else
            Steps2Decel2 = StepsToMove2 - Steps2Stop2;
    } // End Stepper 2 - Calculate steps to target and steps to stop prior to starting motor

    // Stepper 1 - Start running or stopping the motor if commanded and not manually advancing
    if (gStepper_Obj1.RUN_STEPPER == true
            && MOTOR_CMD_ACTIVE1_STEPPER1 == false
            && MOTOR_CMD_ACTIVE2_STEPPER1 == false
            && MOTOR_CMD_ACTIVE3_STEPPER1 == false)
    {
        // START STEPPER 1
        if(gDRV884x_Obj.deviceID == 8849) {
            // case to allow < 160 PPS for DRV8849 only
            if (UseSlowTimerClk1 == false)
                TA0CTL = TASSEL_2 | ID_1 | MC_0;                                         // use 10 MHz clock
            else
                TA0CTL = TASSEL_1 | ID_0 | MC_0;                                         // use 32768 Hz clock
        }

        MOTOR_CMD_ACTIVE1_STEPPER1 = true;                                               // Indicates the motor is running
        LastCommandWasRunSteps1 = true;                                                  // flag to determine if the timer requires clearing
        DesiredStepperSpeed1 = DesiredTargetSpeed1;                                      // Configure the final stepping rate in PPS
        SteppingRate1 = StartingSpeed1;                                                  // This is the starting speed from the GUI

        // adjust for lower step rates
        if (UseSlowTimerClk1 == false)
            SteppingRateTMR1 = SMCLK_FREQ / StartingSpeed1;                              // Convert here to the timer value -- xxTMR value to be loaded into the timer
        else
            SteppingRateTMR1 = SMCLK_LOW_FREQ / StartingSpeed1;                          // Convert here to the timer value -- xxTMR value to be loaded into the timer

        gStepper_Obj1.AccelerateState = ACCEL;                                           // signal to start state machine

        // Timer configuration for DRV8849
        if(gDRV884x_Obj.deviceID == 8849)
        {
            // Configure TimerA0 for STEP1 output
            TA0CCTL1 &= ~OUTMOD_7;                                                       // Clear OUTMODx bits
            TA0CCTL1 |= OUTMOD_3;                                                        // Configure the timer as SET/RESET
            TA0CCTL0 |= CCIE;
            TA0CCR0 = SteppingRateTMR1 - 1;                                              // Start the timer here with initial value
            TA0CCR1 = SteppingRateTMR1 >> 1;                                             // set TBCCR1 to the midpoint of the TBCCR0 for toggle
            // Configure TimerA2 for accel/decel of stepper 1
            TA2CCTL1 &= ~OUTMOD_7;                                                       // Clear OUTMODx bits
            TA2CCTL1 |= OUTMOD_3;                                                        // to SET/RESET
            TA2CCTL0 |= CCIE;                                                            // TIMER_A0 dedicated interrupt
            TA2CCR0 = TIME_32ms;                                                         // set timer to 32ms
            TA2CCR1 = TIME_32ms >> 1;

            // Start timer A2
            if (ok_to_start_timer1 == true)
            {
                TA0CTL  |= TACLR;
                TA2CTL  |= TACLR;
                TA0CTL  |= MC_1;                                                         // start timerA0
                TA2CTL  |= MC_1;                                                         // Start timerA2 by selecting up mode
            }
        }
    } // End Stepper 1 - Start running or stopping the motor if commanded and not manually advancing

    // Stepper 2 - Start running or stopping the motor if commanded and not manually advancing
    if (gStepper_Obj2.RUN_STEPPER == true
            && MOTOR_CMD_ACTIVE1_STEPPER2 == false
            && MOTOR_CMD_ACTIVE2_STEPPER2 == false
            && MOTOR_CMD_ACTIVE3_STEPPER2 == false)
    {
        // START STEPPER
        // case to allow < 160 PPS
        if (UseSlowTimerClk2 == false)
            TBCTL = TBSSEL_2 | ID_1 | MC_0;                                             // use 10 MHz clock
        else
            TBCTL = TBSSEL_1 | ID_0 | MC_0;                                             // use 32768 Hz clock

        MOTOR_CMD_ACTIVE1_STEPPER2 = true;                                              // Indicates the motor is running
        LastCommandWasRunSteps2 = true;                                                 // flag to determine if the timer requires clearing
        DesiredStepperSpeed2 = DesiredTargetSpeed2;                                     // Configure the final stepping rate in PPS
        SteppingRate2 = StartingSpeed2;                                                 // This is the starting speed from the GUI

        // adjust for lower step rates
        if (UseSlowTimerClk2 == false)
            SteppingRateTMR2 = SMCLK_FREQ / StartingSpeed2;                             // Convert here to the timer value -- xxTMR value to be loaded into the timer
        else
            SteppingRateTMR2 = SMCLK_LOW_FREQ / StartingSpeed2;                         // Convert here to the timer value -- xxTMR value to be loaded into the timer

        gStepper_Obj2.AccelerateState = ACCEL;                                          // signal to start state machine
        // Configure TimerB
        TBCCTL1 &= ~OUTMOD_7;                                                           // Clear OUTMODx bits
        TBCCTL1 |= OUTMOD_3;                                                            // Configure the timer as SET/RESET
        TBCCTL0 |= CCIE;
        TBCCR0 = SteppingRateTMR2 - 1;                                                  // Start the timer here with initial value
        TBCCR1 = SteppingRateTMR2 >> 1;                                                 // set TBCCR1 to the midpoint of the TBCCR0 for toggle
        // configure TimerA
        TA1CCTL1 &= ~OUTMOD_7;                                                          // Clear OUTMODx bits
        TA1CCTL1 |= OUTMOD_3;                                                           // to SET/RESET
        TA1CCTL0 |= CCIE;                                                               // TIMER_A0 dedicated interrupt
        TA1CCR0 = TIME_32ms;                                                            // set timer to 32ms
        TA1CCR1 = TIME_32ms >> 1;

        // Start Timer B0
        if (ok_to_start_timer2 == true)
        {
            TB0CTL   |= TBCLR;
            TA1CTL  |= TACLR;
            TB0CTL   |= MC_1;                                                            // start timerB
            TA1CTL  |= MC_1;                                                             // Start timerA by selecting up mode
        }
    } // End Stepper 2 - Start running or stopping the motor if commanded and not manually advancing

    // Stepper 1 is running and STOP is issued
    if ((gStepper_Obj1.RUN_STEPPER == false && MOTOR_CMD_ACTIVE1_STEPPER1 == true))
    {
        // STOP STEPPER
        MOTOR_CMD_ACTIVE1_STEPPER1 = false;                                              // must set low when stopping
        gStepper_Obj1.AccelerateState = DECEL;
        DesiredStepperSpeed1 = StoppingSpeed1;                                           // Speed to stop at
    }

    // MOVE_STEPS
    // Manually advance steps if commanded
    // only allow starting if not automatically running
    if (gStepper_Obj1.MANUAL_ADVANCE == true && 
        MOTOR_CMD_ACTIVE1_STEPPER1 == false && 
        MOTOR_CMD_ACTIVE2_STEPPER1 == false && 
        MOTOR_CMD_ACTIVE3_STEPPER1 == false)
    {
        MOTOR_CMD_ACTIVE2_STEPPER1 = true;
        if (LastCommandWasRunSteps1 == true || use_one_edge == false)                    // added control to avoid resetting
        {
            // Timer configuration for DRV8849
            if(gDRV884x_Obj.deviceID == 8849)
            {
            // reset timer only when starting or when using one edge
            TA0CTL |= TACLR; // reset timerA0 to allow < 160 PPS
            if (UseSlowTimerClk1 == false)
                TA0CTL = CNTL_0 | TASSEL_2 | ID_1 | MC_0;                                // SMCLK/2, 10 MHz
            else
                TA0CTL = CNTL_0 | TASSEL_1 | ID_0 | MC_0;                                // ACLK 32768 Hz

            TA0CCTL1 &= ~OUTMOD_7;                                                       // Clear OUTMODx bits
            TA0CCTL1 |= OUTMOD_3;                                                        // Configure the timer as SET/RESET
            TA0CCTL0 |= CCIE;
            }
        }
        LastCommandWasRunSteps1 = false;                                                 // reset flag to indicate move steps was last commanded
        DesiredStepperSpeed1 = DesiredTargetSpeed1;                                      // Configure the Frequency Rate

        SteppingRate1 = StartingSpeed1;
        if (UseSlowTimerClk1 == false)
            SteppingRateTMR1 = SMCLK_FREQ / StartingSpeed1;                              // Convert here to the timer value -- xxTMR value to be loaded into the timer
        else
            SteppingRateTMR1 = SMCLK_LOW_FREQ / StartingSpeed1;                          // Convert here to the timer value -- xxTMR

        gStepper_Obj1.AccelerateState = ACCEL;                                           // signal to start state machine
        tmpStepsToMove1 = 0;                                                             // clear counter
        MoveSteps1 = true;                                                               // set flag to indicate manual advance

        if(gDRV884x_Obj.deviceID == 8849) {
            // configure TimerA0
            TA0CCR0 = SteppingRateTMR1 - 1;                                              // start the timer
            TA0CCR1 = SteppingRateTMR1 >> 1;                                             // now using count up
            // configure TimerA2
            TA2CCTL1 &= ~OUTMOD_7;                                                       // Clear OUTMODx bits
            TA2CCTL1 |= OUTMOD_3;                                                        // to SET/RESET for 32ms
            TA2CCTL0 |= CCIE;                                                            // TIMER_A0 dedicated interrupt
            TA2CCR0 = TIME_32ms;                                                         // set timer to 32ms
            TA2CCR1 = TIME_32ms >> 1;
            // start timers
            TA0CTL |= MC_1;
            TA2CTL |= MC_1;                                                              // Start timer by selecting up mode
        }
    }// End Stepper 1 is running and STOP is issued

    // Stepper 2 is running and STOP is issued
    if ((gStepper_Obj2.RUN_STEPPER == false && MOTOR_CMD_ACTIVE1_STEPPER2 == true))
    {
        // STOP STEPPER
        MOTOR_CMD_ACTIVE1_STEPPER2 = false;                                             // must set low when stopping
        gStepper_Obj2.AccelerateState = DECEL;
        DesiredStepperSpeed2 = StoppingSpeed2;                                          // Speed to stop at
    }

    // MOVE_STEPS
    // Manually advance steps if commanded
    // only allow starting if not automatically running
    if (gStepper_Obj2.MANUAL_ADVANCE == true
            && MOTOR_CMD_ACTIVE1_STEPPER2 == false
            && MOTOR_CMD_ACTIVE2_STEPPER2 == false
            && MOTOR_CMD_ACTIVE3_STEPPER2 == false)
    {
        MOTOR_CMD_ACTIVE2_STEPPER2 = true;

        if (LastCommandWasRunSteps2 == true || use_one_edge == false)                   // added control to avoid resetting
        {
            // reset timer only when starting or when using one edge
            TB0CTL |= TBCLR;                                                            // reset timerB
            // to allow < 160 PPS
            if (UseSlowTimerClk2 == false)
                TBCTL = TBSSEL_2 | ID_1 | MC_0;                                         // SMCLK/2, 10 MHz
            else
                TBCTL = TBSSEL_1 | ID_0 | MC_0;                                         // ACLK 32768 Hz

            TBCCTL1 &= ~OUTMOD_7;                                                       // Clear OUTMODx bits
            TBCCTL1 |= OUTMOD_3;                                                        // Configure the timer as SET/RESET
            TBCCTL0 |= CCIE;
        }
        LastCommandWasRunSteps2 = false;                                                // reset flag to indicate move steps was last commanded
        DesiredStepperSpeed2 = DesiredTargetSpeed2;                                     // Configure the Frequency Rate

        SteppingRate2 = StartingSpeed2;
        if (UseSlowTimerClk2 == false)
            SteppingRateTMR2 = SMCLK_FREQ / StartingSpeed2;                             // Convert here to the timer value -- xxTMR
        // value to be loaded into the timer
        else
            SteppingRateTMR2 = SMCLK_LOW_FREQ / StartingSpeed2;                         // Convert here to the timer value -- xxTMR

        gStepper_Obj2.AccelerateState = ACCEL;                                          // signal to start state machine
        tmpStepsToMove2 = 0;                                                            // clear counter
        MoveSteps2 = true;                                                              // set flag to indicate manual advance
        // configure Timer B
        TB0CCR0 = SteppingRateTMR2 - 1;                                                 // start the timer
        TB0CCR1 = SteppingRateTMR2 >> 1;                                                // using count up
        // configure TimerA
        TA1CCTL1 &= ~OUTMOD_7;                                                          // Clear OUTMODx bits
        TA1CCTL1 |= OUTMOD_3;                                                           // to SET/RESET for 32ms
        TA1CCTL0 |= CCIE;                                                               // TIMER_A0 dedicated interrupt
        TA1CCR0 = TIME_32ms;                                                            // set timer to 32ms
        TA1CCR1 = TIME_32ms >> 1;
        // start timers
        TBCTL |= MC_1;
        TA1CTL |= MC_1;
    } // End Stepper 2 is running and STOP is issued

    // Reciprocation Stepper 1 during manual advance (STEP control mode) 
    if (gStepper_Obj1.RECIPROCATION == true
            && gStepper_Obj1.MANUAL_ADVANCE == true
            && tmpReciprocation1 == false)
    {
        last_reciprocation_state1 = 1;
        tmpReciprocation1 = true;
        MOTOR_CMD_ACTIVE3_STEPPER1 = true;
        if (LastCommandWasRunSteps1 == true || use_one_edge == false)                    // control to avoid resetting
        {
            if (gDRV884x_Obj.deviceID == 8849)
            {
            // reset timer only when starting or when using one edge
            TA0CTL |= TACLR;                                                             // reset timerA0 for stepper 1

            if (UseSlowTimerClk1 == false)
                TA0CTL = TASSEL_2 | ID_1 | MC_0;
            else
                TA0CTL = TASSEL_1 | ID_0 | MC_0;

            TA0CCTL1 &= ~OUTMOD_7;                                                       // Clear OUTMODx bits
            TA0CCTL1 |= OUTMOD_3;                                                        // Configure the timer as TOGGLE_RESET. TBCCR0 does the reset
            TA0CCTL0 |= CCIE;
            }
        }

        LastCommandWasRunSteps1 = false;                                                 // reset flag to indicate move steps was last commanded
        DesiredStepperSpeed1 = DesiredTargetSpeed1;                                      // Configure the Frequency Rate
        SteppingRate1 = StartingSpeed1;

        if (UseSlowTimerClk1 == false)
            SteppingRateTMR1 = SMCLK_FREQ / StartingSpeed1;                              // Convert here to the timer value -- xxTMR means the value to be loaded into the timer
        else
            SteppingRateTMR1 = SMCLK_LOW_FREQ / StartingSpeed1;                          // Convert here to the timer value -- xxTMR means the value to be loaded into the timer

        gStepper_Obj1.AccelerateState = ACCEL;                                           // signal to start state machine
        tmpStepsToMove1 = 0;                                                             // clear counter
        MoveSteps1 = true;                                                               // set flag to indicate manual advance

        // Configure Timers
        if (gDRV884x_Obj.deviceID == 8849) {
            // configure TimerA0
            TA0CCR0 = SteppingRateTMR1 - 1;                                              // start the timer
            TA0CCR1 = SteppingRateTMR1 >> 1;                                             // now using count up
            // configure TimerA2
            TA2CCTL1 &= ~OUTMOD_7;                                                       // Clear OUTMODx bits
            TA2CCTL1 |= OUTMOD_3;                                                        // to SET/RESET for 32ms
            TA2CCTL0 |= CCIE;                                                            // TIMER_A2 CCR0 dedicated interrupt
            TA2CCR0 = TIME_32ms;                                                         // set timer to 32ms
            TA2CCR1 = TIME_32ms >> 1;
            // start timers
            TA0CTL |= MC_1;
            TA2CTL |= MC_1;                                                              // Start timer by selecting up mode
        }
    } // End Reciprocation Stepper 1 

    // Reciprocation Stepper 2
    if (gStepper_Obj2.RECIPROCATION == true
            && gStepper_Obj2.MANUAL_ADVANCE == true
            && tmpReciprocation2 == false)
    {
        last_reciprocation_state2 = 1;
        tmpReciprocation2 = true;
        MOTOR_CMD_ACTIVE3_STEPPER2 = true;
        if (LastCommandWasRunSteps2 == true || use_one_edge == false)                   // control to avoid resetting
        {
            // reset timer only when starting or when using one edge
            TBCTL |= TBCLR;                                                             // reset timerB to allow < 160 PPS

            if (UseSlowTimerClk2 == false)
                TBCTL = TBSSEL_2 | ID_1 | MC_0;
            else
                TBCTL = TBSSEL_1 | ID_0 | MC_0;

            TBCCTL1 &= ~OUTMOD_7;                                                       // Clear OUTMODx bits
            TBCCTL1 |= OUTMOD_3;                                                        // Configure the timer as TOGGLE_RESET. TBCCR0 does the reset
            TBCCTL0 |= CCIE;
        }
        LastCommandWasRunSteps2 = false;                                                // reset flag to indicate move steps was last commanded
        DesiredStepperSpeed2 = DesiredTargetSpeed2;                                     // Configure the Frequency Rate
        SteppingRate2 = StartingSpeed2;

        if (UseSlowTimerClk2 == false)
            SteppingRateTMR2 = SMCLK_FREQ / StartingSpeed2;                             // Convert here to the timer value -- xxTMR means the value to be loaded into the timer
        else
            SteppingRateTMR2 = SMCLK_LOW_FREQ / StartingSpeed2;                         // Convert here to the timer value -- xxTMR means the value to be loaded into the timer

        gStepper_Obj2.AccelerateState = ACCEL;                                          // signal to start state machine
        tmpStepsToMove2 = 0;                                                            // clear counter
        MoveSteps2 = true;                                                              // set flag to indicate manual advance

        // configure TimerB
        TB0CCR0 = SteppingRateTMR2 - 1;                                                 // start the timer
        TB0CCR1 = SteppingRateTMR2 >> 1;                                                // now using count up

        // configure TimerA
        TA1CCTL1 &= ~OUTMOD_7;                                                          // Clear OUTMODx bits
        TA1CCTL1 |= OUTMOD_3;                                                           // to SET/RESET for 32ms
        TA1CCTL0 |= CCIE;                                                               // TIMER_A1 CCR0 dedicated interrupt
        TA1CCR0 = TIME_32ms;                                                            // set timer to 32ms
        TA1CCR1 = TIME_32ms >> 1;

        // start timers
        TBCTL |= MC_1;
        TA1CTL |= MC_1;                                                                 // Start timer by selecting up mode
    } // End Reciprocation Stepper 2

    // Stepper 1 Special case of updating speed during running. If the Desired Target Speed has been updated, then the stepper will be increased or decreased
    if ((gStepper_Obj1.RUN_STEPPER == true)  &&  (DesiredStepperSpeed1 != DesiredTargetSpeed1))
    {
        if (DesiredTargetSpeed1 > DesiredStepperSpeed1)
            gStepper_Obj1.AccelerateState = ACCEL;
        else
            gStepper_Obj1.AccelerateState = DECEL2NEWTARGET;

        DesiredStepperSpeed1 = DesiredTargetSpeed1;                                      // reset DesiredStepperSpeed
    } // End Stepper 1 Special case 

    // Stepper 2 Special case of updating speed during running. If the Desired Target Speed has been updated, then the stepper will be increased or decreased
    if ((gStepper_Obj2.RUN_STEPPER == true)  &&  (DesiredStepperSpeed2 != DesiredTargetSpeed2))
    {
        if (DesiredTargetSpeed2 > DesiredStepperSpeed2)
            gStepper_Obj2.AccelerateState = ACCEL;
        else
            gStepper_Obj2.AccelerateState = DECEL2NEWTARGET;

        DesiredStepperSpeed2 = DesiredTargetSpeed2;                                     // reset DesiredStepperSpeed
    } // End Stepper 2 Special case

}

// Stepper State machine (This is used to move the motor)
void MotorStateMachine_8849()
{
    // Stepper 1 - update the SteppingRateTMR1 if required
    if (UpdateSteppingRateTMR1 == true) // Reduce the number of calculations
    {
        // automatically select timer clock source based on stepping rate to allow very low as well as very high stepping rates
        if(SteppingRate1 > 160 && UseSlowTimerClk1 == true)
            UseSlowTimerClk1 = false;
        else if(SteppingRate1 < 160 && UseSlowTimerClk1 == false)
            UseSlowTimerClk1 = true;

        // recalculate stepping rate update in case Acceleration Rate was updated (integer value divide by 32)
        AccelRate1 = (unsigned long) gStepper_Obj1.ACCEL_RATE << use_one_edge;           // Configure the acceleration rate in case Acceleration Rate was updated

        if (AccelRate1 > 31 || AccelRate1 == 0)                                          // if >=32 or 0
            SteppingRateUpdate1 = AccelRate1 >> 5;                                       // divide by 32 by right shifting 5 positions
        else
            SteppingRateUpdate1 = 1;                                                     // set it to 1 to get correct value

        // setup TimerA0 clock
        if (UseSlowTimerClk1 == false)
        {
            TA0CTL &= ~(ID_3 + TASSEL_1);
            TA0CTL |=   ID_1 + TASSEL_2;                                                 // use 10 MHz clock
            SteppingRateTMR1 = SMCLK_FREQ / SteppingRate1;                               // Convert here to the timer value -- xxTMR means the value to be loaded into the timer
        }
        else
        {
            TA0CTL &= ~(ID_3 + TASSEL_2);
            TA0CTL |=   ID_0 + TASSEL_1;                                                 // use 32768 Hz clock
            SteppingRateTMR1 = SMCLK_LOW_FREQ / SteppingRate1;                           // Convert here to the timer value -- xxTMR means the value to be loaded into the timer
        } // End Stepper 1 - update the SteppingRateTMR1 if required

        UpdateSteppingRateTMR1 = false;                                                  // clear flag
    }

    // Stepper 1 Accelerate State Control
    switch (gStepper_Obj1.AccelerateState)
    {
    case (IDLE):
            if (!(gStepper_Obj1.RECIPROCATION) && gDRV884x_Obj.nSleep && gDRV884x_Obj.gDRVEN1)
            {
                MOTOR_CMD_ACTIVE1_STEPPER1 = false;
                MOTOR_CMD_ACTIVE2_STEPPER1 = false;
                MOTOR_CMD_ACTIVE3_STEPPER1 = false;
                gStepper_Obj1.MANUAL_ADVANCE = false;                                    // clear the flags once stops
                MoveSteps1 = false;                                                      // clear the MoveSteps1 flag here

                // reset the timers if the last command was run motor
                if (LastCommandWasRunSteps1)
                {
                    TA0CTL &= ~MC_3;                                                     // Stop Timer_A0
                    TA0CTL |= TACLR;
                    TA2CTL &= ~MC_3;                                                     // Stop Timer_A1
                    TA2CTL |= TACLR;
                }
            }
            break;
    case (ACCEL):                                                                        // do nothing in ACCEL mode, wait for external input to change state
            break;
    case (DECEL):
            break;
    case (RUN):
            break;
    case (DECEL2NEWTARGET):
            break;
    } // End Stepper 1 Accelerate State Control

    // Stepper 2 - update the SteppingRateTMR1 if required
    if (UpdateSteppingRateTMR2 == true)                                                  // Reduce the number of calculations
    {
        // automatically select timer clock source based on stepping rate to allow very low as well as very high stepping rates
        if(SteppingRate2 > 160 && UseSlowTimerClk2 == true)
            UseSlowTimerClk2 = false;
        else if(SteppingRate2 < 160 && UseSlowTimerClk2 == false)
            UseSlowTimerClk2 = true;

        // recalculate stepping rate update in case Acceleration Rate was updated (integer value divide by 32)
        AccelRate2 = (unsigned long) gStepper_Obj2.ACCEL_RATE << use_one_edge;           // Configure the acceleration rate in case Acceleration Rate was updated

        if (AccelRate2 > 31 || AccelRate2 == 0)                                          // if >=32 or 0
            SteppingRateUpdate2 = AccelRate2 >> 5;                                       // divide by 32 by right shifting 5 positions
        else
            SteppingRateUpdate2 = 1;                                                     // set it to 1 to get correct value

        // setup Timer_B clock
        if (UseSlowTimerClk2 == false)
        {
            TB0CTL &= ~(ID_3 + TBSSEL_1);
            TB0CTL |= ID_1 + TBSSEL_2;                                                   // use 10 MHz clock
            SteppingRateTMR2 = SMCLK_FREQ / SteppingRate2;                               // Convert here to the timer value -- xxTMR means the value to be loaded into the timer
        }
        else
        {
            TB0CTL &= ~(ID_3 + TBSSEL_2);
            TB0CTL |= ID_0 + TBSSEL_1;                                                   // use 32768 Hz clock
            SteppingRateTMR2 = SMCLK_LOW_FREQ / SteppingRate2;                           // Convert here to the timer value -- xxTMR means the value to be loaded into the timer
        }

        UpdateSteppingRateTMR2 = false;                                                  // clear flag
    } // End Stepper 2 - update the SteppingRateTMR2 if required

    // Stepper 2 Accelerate State Control
    switch (gStepper_Obj2.AccelerateState)
    {
    case (IDLE):
            if (!(gStepper_Obj2.RECIPROCATION) && gDRV884x_Obj.nSleep && gDRV884x_Obj.gDRVEN2)
            {
                MOTOR_CMD_ACTIVE1_STEPPER2 = false;
                MOTOR_CMD_ACTIVE2_STEPPER2 = false;
                MOTOR_CMD_ACTIVE3_STEPPER2 = false;
                gStepper_Obj2.MANUAL_ADVANCE = false;                                    // clear the flags once stops
                MoveSteps2 = false;                                                      // clear the MoveSteps flag here

                // reset the timers if the last command was run motor
                if (LastCommandWasRunSteps2)
                {
                    TB0CTL &= ~MC_3;                                                     // Stop Timer_B
                    TB0CTL |= TBCLR;
                    TA1CTL &= ~MC_3;                                                     // Stop Timer_A0
                    TA1CTL |= TACLR;
                }
            }
            break;
    case (ACCEL):                                                                        // do nothing in ACCEL mode, wait for external input to change state
            break;
    case (DECEL):
            break;
    case (RUN):
            break;
    case (DECEL2NEWTARGET):
            break;
    } // End Stepper 2 Accelerate State Control
}

void SetMotorParameters_8849()
{   
    // Stepper 1 Update Motor Parameters
    if (UpdateMotorParameters1 == true)
    {
        // increment or decrement to the new timer A value
        if (gStepper_Obj1.AccelerateState == ACCEL && (SteppingRate1 <= DesiredTargetSpeed1))
        {
            // increase pps and calculate new pps
            if ((SteppingRate1 + SteppingRateUpdate1) >= DesiredTargetSpeed1)
            {
                gStepper_Obj1.AccelerateState = RUN;                                     // No longer accelerate
                SteppingRate1 = DesiredTargetSpeed1;
            }
            else
            {
                SteppingRate1 += SteppingRateUpdate1;                                    // Increment
                gStepper_Obj1.AccelerateState = ACCEL;
            }
        }
        if (gStepper_Obj1.AccelerateState == DECEL)
        {
            if (SteppingRate1 >= StoppingSpeed1)
            {
                if (SteppingRate1 <= (StoppingSpeed1 + SteppingRateUpdate1))
                {
                    if (MoveSteps1 == false && SteppingRate1 == StoppingSpeed1)
                        gStepper_Obj1.AccelerateState = IDLE;                            // Stop motor when speed is reached
                    SteppingRate1 = StoppingSpeed1;
                }
                else
                {
                    SteppingRate1 -= SteppingRateUpdate1;                                // Decrement
                    gStepper_Obj1.AccelerateState = DECEL;
                }
            }
            else                                                                         // already less than stopping speed
            {
                if (MoveSteps1 == false)
                    gStepper_Obj1.AccelerateState = IDLE;                                // Stop motor when speed is reached
            }

        }

        // this allows for the motor to spin down to new speed without stopping
        if (gStepper_Obj1.AccelerateState == DECEL2NEWTARGET)
        {
            if (SteppingRate1 >= DesiredTargetSpeed1)
            {
                // decrease pps and calculate new pps
                if ((SteppingRate1 - SteppingRateUpdate1) <= DesiredTargetSpeed1)
                {
                    if (SteppingRate1 == DesiredTargetSpeed1)
                    {
                        gStepper_Obj1.AccelerateState = RUN;                             // switch to RUN when target speed is reached
                    }
                    SteppingRate1 = DesiredTargetSpeed1;
                }
                else
                {
                    SteppingRate1 -= SteppingRateUpdate1;                                // Decrement
                    gStepper_Obj1.AccelerateState = DECEL2NEWTARGET;
                }
            }
        }

        // Update the stepping rate. If the stepping rate has been updated, set a flag
        if (SteppingRate1 != PriorSteppingRate1)
        {
            UpdateSteppingRateTMR1 = true;                                               // Set flag to re-calculate stepping rate TMR
        }

        UpdateMotorParameters1 = false;                                                  // clear the flag
    } // End Stepper 1 Update Motor Parameters

    // Stepper 2 Update Motor Parameters
    if (UpdateMotorParameters2 == true)
    {
        // increment or decrement to the new timer A value
        if (gStepper_Obj2.AccelerateState == ACCEL && (SteppingRate2 <= DesiredTargetSpeed2))
        {
            // increase pps and calculate new pps
            if ((SteppingRate2 + SteppingRateUpdate2) >= DesiredTargetSpeed2)
            {
                gStepper_Obj2.AccelerateState = RUN;                                     // No longer accelerate
                SteppingRate2 = DesiredTargetSpeed2;
            }
            else
            {
                SteppingRate2 += SteppingRateUpdate2;                                    // Increment
                gStepper_Obj2.AccelerateState = ACCEL;
            }
        }
        if (gStepper_Obj2.AccelerateState == DECEL)
        {
            if (SteppingRate2 >= StoppingSpeed2)
            {
                if (SteppingRate2 <= (StoppingSpeed2 + SteppingRateUpdate2))
                {
                    if (MoveSteps2 == false && SteppingRate2 == StoppingSpeed2)
                        gStepper_Obj2.AccelerateState = IDLE;                            // Stop acceleration when speed is reached
                    SteppingRate2 = StoppingSpeed2;
                }
                else
                {
                    SteppingRate2 -= SteppingRateUpdate2;                                // Decrement
                    gStepper_Obj2.AccelerateState = DECEL;
                }
            }
            else                                                                         // already less than stopping speed
            {
                if (MoveSteps2 == false)
                    gStepper_Obj2.AccelerateState = IDLE;                                // Stop acceleration when speed is reached
            }

        }

        // this allows for the motor to spin down to new speed without stopping
        if (gStepper_Obj2.AccelerateState == DECEL2NEWTARGET)
        {
            if (SteppingRate2 >= DesiredTargetSpeed2)
            {
                // decrease pps and calculate new pps
                if ((SteppingRate2 - SteppingRateUpdate2) <= DesiredTargetSpeed2)
                {
                    if (SteppingRate2 == DesiredTargetSpeed2)
                    {
                        gStepper_Obj2.AccelerateState = RUN;                             // switch to RUN when target speed is reached
                    }
                    SteppingRate2 = DesiredTargetSpeed2;
                }
                else
                {
                    SteppingRate2 -= SteppingRateUpdate2;                                // Decrement
                    gStepper_Obj2.AccelerateState = DECEL2NEWTARGET;
                }
            }
        }

        // Update the stepping rate. If the stepping rate has been updated, set a flag
        if (SteppingRate2 != PriorSteppingRate2)
        {
            UpdateSteppingRateTMR2 = true;                                               // Set flag to re-calculate stepping rate TMR
        }
        UpdateMotorParameters2 = false;                                                  // clear the flag
    } // End Stepper 2 Update Motor Parameters
}

void DetermineNextAction_8849()
{
    // Stepper 1 Update Next Action
    if (UpdateNextAction1)
    {
        if (MoveSteps1)                                                                  // if manual advance count steps
        {                                                                                // when complete
            if (tmpStepsToMove1 >= Steps2Decel1)
            {
                gStepper_Obj1.AccelerateState = DECEL;                                   // start deceleration
            }

            if (tmpStepsToMove1 >= StepsToMove1)                                         // stop here when equal
            {
                gStepper_Obj1.AccelerateState = IDLE;
                tmpReciprocation1 = false;

                if ((gStepper_Obj1.RECIPROCATION && gStepper_Obj1.MANUAL_ADVANCE))
                {
                    tmpStepsToMove1 = 0;                                                 // clear counter
                    gStepper_Obj1.gDIR = nextDIR1;
                }

                if (!(gStepper_Obj1.RECIPROCATION))
                {
                    if (last_reciprocation_state1)
                    {
                        gStepper_Obj1.gDIR = nextDIR1;
                        last_reciprocation_state1 = 0;
                    }
                    MoveSteps1 = false;
                }

                TA0CTL &= ~MC_3;                                                         // Stop TimerA0
                // TA0CCTL3 &= ~(OUTMOD_7 + OUT);                                        // Output low
                TA0CTL |= TBCLR;
                TA2CTL &= ~MC_3;                                                         // Stop TimerA2
                TA2CTL |= TACLR;
            }
        }

        UpdateNextAction1 = false;                                                       // clear the flag
    } // End Stepper 1 Update Next Action

    // Stepper 2 Update Next Action
    if (UpdateNextAction2)
    {
        if (MoveSteps2)                                                                  // if manual advance count steps
        {                                                                                // when complete
            if (tmpStepsToMove2 >= Steps2Decel2)
            {
                gStepper_Obj2.AccelerateState = DECEL;                                   // start deceleration
            }

            if (tmpStepsToMove2 >= StepsToMove2)                                         // stop here when equal
            {
                gStepper_Obj2.AccelerateState = IDLE;
                tmpReciprocation2 = false;

                if ((gStepper_Obj2.RECIPROCATION && gStepper_Obj2.MANUAL_ADVANCE))
                {
                    tmpStepsToMove2 = 0;                                                 // clear counter
                    gStepper_Obj2.gDIR = nextDIR2;
                }

                if (!(gStepper_Obj2.RECIPROCATION))
                {
                    if (last_reciprocation_state2)
                    {
                        gStepper_Obj2.gDIR = nextDIR2;
                        last_reciprocation_state2 = 0;
                    }
                    MoveSteps2 = false;
                }

                TB0CTL &= ~MC_3;                                                         // Stop Timer_B
                //TBCCTL3 &= ~(OUTMOD_7 + OUT);                                            // Output low
                TB0CTL |= TBCLR;
                TA1CTL &= ~MC_3;                                                         // Stop Timer_A
                TA1CTL |= TACLR;
            }
        }
    
        UpdateNextAction2 = false;                                                       // clear the flag
    } // End Stepper 2 Update Next Action
}
