/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  stepper.h - Declaration file for functions and global variables related to stepper control
 *  DRV884x_EVM_FIRMWARE
 *  08/25/2022
 *  Jacob Thompson
 *
 ******************************************************************************/
#ifndef STEPPER_H_
#define STEPPER_H_
/******************************************************************************
 * HEADER FILES
 *****************************************************************************/
#include <dataTypeDefinition.h>
#include "drv884x/include/drv8845.h"
#include "drv884x/include/utility.h"
#include <msp430.h>
#include <stepper.h>

/******************************************************************************
 * Defined Quantities
 *****************************************************************************/
#define LED_TIMEOUT 125                     // Count to blink LED with 1s period
#define SMCLK_FREQ  10000000                // SMCLK frequency 10 MHz
#define SMCLK_LOW_FREQ 32768                // used for stepping rates below 160 TBCLK = 32768 Hz
#define TIME_32ms   10001                   // TimerA interrupt value, (32ms or 31.25 updates/s) at 2.5MHz/8

// Stepper 1 and 2 default settings
#define DEFAULT_START_SPEED     200         // initial speed in pulses per second when starting motor
#define DEFAULT_TARGET_SPEED    1000        // speed of motor once running
#define DEFAULT_ACCEL_RATE      1000        // acceleration rate from starting speed to target speed
#define DEFAULT_STOPPING_SPEED  200         // speed to stop motor once deceleration has reached this point
#define DEFAULT_NUM_STEPS       1000        // number of steps to move when manually stepping
#define DEFAULT_STEPS_2_STOP    1           // number of steps used to stop the motor when manually stepping
#define DEFAULT_DecayMode       0           // Smart tune ripple control
#define DEFAULT_STEP_MODE_8849  6           // 1/16 uStep
#define DEFAULT_FSCURRENT_8849  0.5         // 0.5A
#define DEFAULT_IFS             0.5         // full scale current setting
#define use_one_edge            0           // multiplies all step calculations by 1
#define Lower_12_bits           0xFFF

// defines for motor state machine
#define IDLE            1
#define ACCEL           2
#define DECEL           4
#define RUN             8
#define DECEL2NEWTARGET 16

/*****************************************************************************/
// Declare Global Variables

//***********************************************************************************
// Global Variables for GUI Composer                                                         *
//***********************************************************************************

typedef struct Stepper_Driving_Obj
{
   bool RUN_STEPPER;                                       // Auto run enabled
   bool MANUAL_ADVANCE;                                    // Manually advance motor
   bool RECIPROCATION;
   uint8_t  gDIR;                                          // Direction control, 0 = Forward and 1 = Reverse

   uint16_t STARTING_SPEED;                                // initial speed in pulses per second when starting motor
   uint16_t TARGET_SPEED;                                  // speed of motor once running
   uint16_t ACCEL_RATE;                                    // acceleration rate from starting speed to target speed
   uint16_t STOPPING_SPEED;                                // speed to stop motor once deceleration has reached this point
   uint16_t STEPS_2_STOP;                                  // number of steps used to stop the motor when manually stepping
   uint16_t NUM_STEPS;                                     // number of steps to move when manually stepping
   uint16_t AccelerateState;                               // Accelerate state for state machine
   uint16_t StallState;                                    // Stall detection setup state

   float IFS;                                              // Full scale current
} Stepper_Driving_Obj_t;

extern volatile Stepper_Driving_Obj_t gStepper_Obj1;       // for Stepper 1
extern volatile Stepper_Driving_Obj_t gStepper_Obj2;       // for Stepper 2

/////////////////////////
// variables for utility
/////////////////////////

// variable for the motor state machine
unsigned long int SteppingRateTMR1;
unsigned long int SteppingRate1;
unsigned long int SteppingRateUpdate1;
unsigned long int DesiredStepperSpeed1;
unsigned long int DesiredTargetSpeed1;
unsigned long int StartingSpeed1;
unsigned long int StoppingSpeed1;
unsigned long int AccelRate1;
unsigned long int StepsToStop1;
unsigned long int StepsToMove1;
unsigned long int tmpStepsToMove1;
unsigned long int PriorSteppingRate1;

unsigned long int SteppingRateTMR2;
unsigned long int SteppingRate2;
unsigned long int SteppingRateUpdate2;
unsigned long int DesiredStepperSpeed2;
unsigned long int DesiredTargetSpeed2;
unsigned long int StartingSpeed2;
unsigned long int StoppingSpeed2;
unsigned long int AccelRate2;
unsigned long int StepsToStop2;
unsigned long int StepsToMove2;
unsigned long int tmpStepsToMove2;
unsigned long int PriorSteppingRate2;

uint8_t nextDIR1;                                          // for Stepper 1
uint8_t nextDIR2;                                          // for Stepper 2

// For DRV8845
bool MOTOR_CMD_ACTIVE1_STEPPER;                            // Motor in Motion (Auto)   0 - No Motion   1 - In Motion
bool MOTOR_CMD_ACTIVE2_STEPPER;                            // Motor in Motion (Man)    0 - No Motion   1 - In Motion
bool MOTOR_CMD_ACTIVE3_STEPPER;

// For DRV8849
bool MOTOR_CMD_ACTIVE1_STEPPER1;                           // Motor in Motion (Auto)   0 - No Motion   1 - In Motion
bool MOTOR_CMD_ACTIVE2_STEPPER1;                           // Motor in Motion (Man)    0 - No Motion   1 - In Motion
bool MOTOR_CMD_ACTIVE3_STEPPER1;
bool enable_Stepper1Timer_Interrupt;                       // Timer A0 Interrupt for Stepper 1
bool UpdateMotorParameters1;
bool UpdateNextAction1;
bool tmpReciprocation1;                                    // NEW, NEED TO INITIALIZE ETC
bool tmpDelay1;                                            // NEW, NEED TO INITIALIZE ETC
bool LastCommandWasRunSteps1;
bool MoveSteps1;
bool UpdateSteppingRateTMR1;
bool UseSlowTimerClk1;
bool ok_to_start_timer1;                                   // used to block timer start in some cases

bool MOTOR_CMD_ACTIVE1_STEPPER2;                           // Motor in Motion (Auto)   0 - No Motion   1 - In Motion
bool MOTOR_CMD_ACTIVE2_STEPPER2;                           // Motor in Motion (Man)    0 - No Motion   1 - In Motion
bool MOTOR_CMD_ACTIVE3_STEPPER2;
bool enable_Stepper2Timer_Interrupt;                       // Timer B0 Interrupt for Stepper 2
bool UpdateMotorParameters2;
bool UpdateNextAction2;
bool tmpReciprocation2;
bool tmpDelay2;
bool LastCommandWasRunSteps2;
bool MoveSteps2;
bool UpdateSteppingRateTMR2;
bool UseSlowTimerClk2;
bool ok_to_start_timer2;                                   // used for blocking timer start


float time1;
float time_squared1;
float time2;
float time_squared2;
float IFS_previous1;                                       // Previous full-scale current setting for Stepper 1
float IFS_previous2;                                       // Previous full-scale current setting for Stepper 2
unsigned int last_reciprocation_state1;                    // reciprocation flag, =1 last action is reciprocation
unsigned int last_reciprocation_state2;                    // reciprocation flag, =1 last action is reciprocation

unsigned int LED_TOGGLE_CNT;                               // count to determine if LED should be toggled
unsigned int clear_fault_loop;                             // counter for clear fault pulse

unsigned long int Steps2Target1;                           // number of steps to reach target speed from starting speed using acceleration rate
unsigned long int Steps2Stop1;                             // number of steps to stop from target speed using acceleration rate
unsigned long int Steps2Decel1;                            // step number to change from accel to decel in manual mode

unsigned long int Steps2Target2;                           // number of steps to reach target speed from starting speed using acceleration rate
unsigned long int Steps2Stop2;                             // number of steps to stop from target speed using acceleration rate
unsigned long int Steps2Decel2;                            // step number to change from accel to decel in manual mode

/*****************************************************************************/

// Function Declarations

// Main Functions
void UpdateDeviceControls_Stepper_DRV8849();
void UpdateGUIControls_DRV8849();
void UpdateSpeed_8849();

// GPIO/GUI Functions
void stepper_8849_Init();

// Motor Functions
void DetermineMotorMovement_8849();
void MotorStateMachine_8849();
void SetMotorParameters_8849();
void DetermineNextAction_8849();

#endif /*STEPPER_H_*/
