/* --COPYRIGHT--,BSD
 * Copyright (c) 2016,2019, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/********************************************************************************
 *
 *  stepper.c - Definition file for functions related to stepper motion control
 *  DRV8845_Example FIRMWARE
 *  10/28/2020
 *  updated 03/01/2023
 *  Murugavel Raju
 *
 *******************************************************************************/

/********************************************************************************
 * HEADER FILES
 *******************************************************************************/
#include "stepper_8845.h"
#include "stepper.h"

/********************************************************************************
 * LOCAL ROUTINES
 *******************************************************************************/

void stepper_8845_Init(void)
{
    // Reset Timer B0 and A0 from DRV8845 Brushed setup
    TB0CTL &= ~0xFFFF;                                               // Reset TB0 configuration
    TB0CCTL0 &= 0xFDF7;                                              // Reset TB0CCTL0
    TB0CCTL2 &= 0xFDF7;                                              // Reset TB0CCTL2
    TB0CCTL4 &= 0xFDF7;                                              // Reset TB0CCTL4
    TB0CCR0 = 0;                                                     // Clear TBOCCR0

    TA0CTL &= ~0xFFFF;                                               // Reset TA0 configuration
    TA0CCTL0 &= 0xF9F7;                                              // Reset TA0CCTL0
    TA0CCTL2 &= 0xF9F7;                                              // Reset TA0CCTL2
    TA0CCTL4 &= 0xF9F7;                                              // Reset TA0CCTL4
    TA0CCR0 = 0;                                                     // Clear TAOCCR0

    // configure Timer A1 for DRV8845 Stepper 1 output
    TA1CTL = TASSEL_2 | ID_3 | MC_0;                                 // divide /8 for 2.5 MHz

    // configure Timer B0 for acceleration and deceleration control for DRV8845 Stepper 1
    TBEX0 |= TBIDEX_1;                                               // divide /2 for 10 MHz
    TB0CTL = TBCLGRP_0 | TBSSEL_2 | ID_3 | MC_0;                     // divide /8 for 1.25 MHz

    // REF configuration for 1.5V VREF for DAC12_x
    REFCTL0 |= REFON;                                                // Internal reference VREF on

    // DAC12 configuration
    DAC12_0CTL0 =  DAC12CALON | DAC12SREF_0 | DAC12IR | DAC12AMP_7;  // Configure with VREF=1.5V DAC12_0 on P6.6 initiate DAC12_1 calibration
    while (DAC12_0CTL0 & DAC12CALON);                                // Wait for Calibration complete
    DAC12_0DAT = 0;                                                  // Default output 0V

    DAC12_1CTL0 =  DAC12CALON | DAC12SREF_0 | DAC12IR | DAC12AMP_7;  // Configure with VREF=1.5V DAC12_1 on P6.7 initiate DAC12_1 calibration
    while (DAC12_1CTL0 & DAC12CALON);                                // Wait for Calibration complete
    DAC12_1DAT = 0;                                                  // Default output 0V
    //End DAC12 configuration

    // Port P1 re-configuration
    P1OUT = 0;                                                       // All outputs low
    P1SEL = 0;                                                       // Clear P1SEL
    P1DIR = BIT7 | BIT6 | I02 | PHASE2 | I01 | PHASE1 | BIT1 | BIT0; // All are outputs

    // Port P4 configuration
    P4OUT = 0;                                                       // All outputs low
    P4SEL = 0;                                                       // Clear P4SEL
    P4DIR = BIT7 | BIT6 | BIT5 | I04 | PHASE4 | I03 | PHASE3 | BIT0; // All are outputs

    set_I01_I11_hi;                                                  // set output current to 0
    set_I02_I12_hi;
    set_ENABLE1_led_lo;

    set_I03_I13_hi;                                                  // set output current to 0
    set_I04_I14_hi;
    set_ENABLE2_led_lo;

    // Set Default GUI Variables Values
    SetDefaultGUIVals();
    
    // set boolean defaults
	last_reciprocation_state = 0;
	LastCommandWasRunSteps = false;
	MoveSteps = false;
	UpdateSteppingRateTMR = false;
	temp_reciprocation = false;
    MOTOR_CMD_ACTIVE1_STEPPER = false;
    MOTOR_CMD_ACTIVE2_STEPPER = false;
    MOTOR_CMD_ACTIVE3_STEPPER = false;

	// set initial value for the VREF DACs 71% phase currents
    Next_a_value = 1444;
    Next_b_value = 1444;

	// Set initial values
	Next_a_step 	= 272;
	Next_b_step 	= 784;
	prev_gDRVEN1  	= 0;
	prev_gDRVEN2    = 0;
	temp_step_mode 	= 0;
	tmp_ichop_val 	= 0;
}

void UpdateDeviceControls_Stepper_DRV8845()
{
    // toggle MCU_STATUS LED if count is reached
    if (LED_TOGGLE_CNT > LED_TIMEOUT)
    {
        toggle_MCUstatus_led;
        LED_TOGGLE_CNT = 0;

        // Measure VM and update to GUI display
        ADC12CTL0 = ADC12ON;                                         // Enable ADC
        ADC12IE = 0;                                                 // Disable ADC interrupts
        ADC12CTL1 = ADC12SHP;                                        // configure for single conversion
        ADC12MCTL0 = ADC12INCH_0 + ADC12SREF_0;                      // A0 for VM measurement
        ADC12CTL0 |= ADC12SC + ADC12ENC;                             // Enable and start conversion
        while (ADC12CTL1 & ADC12BUSY);                               // Wait for conversion to complete
        gDRV884x_Obj.VMvoltage = ADC12MEM0;                          // Conversion result stored in ADC12MEM0 sent to GUI display
        if (gDRV884x_Obj.VMvoltage < ADC12scale)
            gDRV884x_Obj.VMvoltage = 0;                              // VM is 0
        ADC12CTL0 &= ~ADC12ENC;                                      // Disable ADC
        // End measure VM and update to GUI display
    }

    // Full scale current or ICHOP value update, value of 0 to 16 is translated to various full scale current percentages
    if (tmp_ichop_val != ichop_val)
    {
        tmp_ichop_val = ichop_val;
    }

    // Step Mode settings update for Stepper. Step mode sets the increments between sinewave.h values
    if (temp_step_mode != StepMode)
    {
        if (StepMode > 1)
            step_offset = 512 >> (StepMode - 1);
        else
            step_offset = 512 >> StepMode;

        temp_step_mode = StepMode;
    }

    // nSLEEP control
    if (gDRV884x_Obj.nSleep && !(read_nSLEEP_pin))
    {
        set_nSLEEP_hi;
        _delay_cycles(30000);                                        // 1.5 ms wake up delay

        // set initial value for the VREF DACs phase currents
        Next_a_value = 1444;
        Next_b_value = 1444;

        // Set initial indexer position
        Next_a_step     = 272;
        Next_b_step     = 784;
    }
    else if (!gDRV884x_Obj.nSleep && (read_nSLEEP_pin))
    {
        gStepper_Obj1.RUN_STEPPER       = false;
        gStepper_Obj2.RUN_STEPPER       = false;
        gStepper_Obj1.MANUAL_ADVANCE    = false;
        gStepper_Obj2.MANUAL_ADVANCE    = false;
        gDRV884x_Obj.gDRVEN1            = 0;
        gDRV884x_Obj.gDRVEN2            = 0;
        __delay_cycles(100000);

        // stop the timers
        TB0CTL &= ~MC_3;
        TB0CTL |= TACLR;
        TA0CTL &= ~MC_3;
        TA0CTL |= TACLR;
        TA1CTL &= ~MC_3;
        TA1CTL |= TACLR;
        TA2CTL &= ~MC_3;
        TA2CTL |= TACLR;

        set_nSLEEP_lo;
        set_ENABLE1_led_lo;
        set_ENABLE2_led_lo;
    }

    // Clear Fault
    if (gDRV884x_Obj.ClrFlt)
    {
        ClearFaultPulse();
        gDRV884x_Obj.ClrFlt = 0;
    }

    // Set flags to determine stepper motors Enabled and set the EN and PH pins for the default values
	if (gDRV884x_Obj.gDRVEN1 != prev_gDRVEN1 || gDRV884x_Obj.gDRVEN2 != prev_gDRVEN2)
    {
		if (gDRV884x_Obj.gDRVEN1 || gDRV884x_Obj.gDRVEN2)  // Either Stepper 1 or Stepper 2 Enabled
        {
		    // Set the initial indexer DAC values for default uStep 1/16 setting and default current setting
			AVREF_DAC = Next_a_value >> 2;                 // AVREF default setting
			BVREF_DAC = Next_b_value >> 2;                 // BVREF default setting

			if (gDRV884x_Obj.gDRVEN1)
			{
	            // Initialize I01, 11, 02 and 12 to enable (LOW - LOW) to hold stepper
	            set_I01_I11_lo;                            // set I01 and I11 LOW, pins are tied together
	            set_I02_I12_lo;                            // set I02 and I12 LOW, pins are tied together
	            set_phase1_lo;
	            set_phase2_hi;
	            set_ENABLE1_led_hi;
			}

            if (gDRV884x_Obj.gDRVEN2)
            {
                // Initialize I03, 13, 04 and 14 to enable (LOW - LOW) to hold stepper
                set_I03_I13_lo;                            // set I03 and I13 LOW, pins are tied together
                set_I04_I14_lo;                            // set I04 and I14 LOW, pins are tied together
                set_phase3_lo;
                set_phase4_hi;
                set_ENABLE2_led_hi;
            }
        }
		else if (!gDRV884x_Obj.gDRVEN1 && !gDRV884x_Obj.gDRVEN2) // Stepper 1 and Stepper 2 Disabled
		{
            AVREF_DAC = 0;                                 // AVREF
            BVREF_DAC = 0;                                 // BVREF
		}

		if (!gDRV884x_Obj.gDRVEN1)                         // Output Disabled
		{
		    set_I01_I11_hi;                                // Initialize to disable (HIGH - HIGH)
		    set_I02_I12_hi;
		    set_ENABLE1_led_lo;
		}

        if (!gDRV884x_Obj.gDRVEN2)                         // Output Disabled
        {
            set_I03_I13_hi;                                // Initialize to disable (HIGH - HIGH)
            set_I04_I14_hi;
            set_ENABLE2_led_lo;
        }

        prev_gDRVEN1 = gDRV884x_Obj.gDRVEN1;
        prev_gDRVEN2 = gDRV884x_Obj.gDRVEN2;
	}

    // Direction control for Stepper 1 and 2
    if (Direction  && next_Direction)
    {
        if (!Reciprocate && RunStepper)
        {
            // stop the timers
            TB0CTL &= ~MC_3;
            TB0CTL |= TACLR;
            TA1CTL &= ~MC_3;
            TA1CTL |= TACLR;
            RunStepper = false;
            MOTOR_CMD_ACTIVE1_STEPPER = false;
            MOTOR_CMD_ACTIVE2_STEPPER = false;
            MOTOR_CMD_ACTIVE3_STEPPER = false;
            __delay_cycles(20000);                                   // 1ms delay
            Direction = 1;
            next_Direction = 0;
            RunStepper = true;
        }
        else
        {
            Direction = 1;
            next_Direction = 0;
        }
    }
    else if (!Direction  && !next_Direction)
    {
        if (!Reciprocate && RunStepper)
        {
            // stop the timers
            TB0CTL &= ~MC_3;
            TB0CTL |= TACLR;
            TA1CTL &= ~MC_3;
            TA1CTL |= TACLR;
            RunStepper = false;
            MOTOR_CMD_ACTIVE1_STEPPER = false;
            MOTOR_CMD_ACTIVE2_STEPPER = false;
            MOTOR_CMD_ACTIVE3_STEPPER = false;
            Direction = 0;;
            next_Direction = 1;
            RunStepper = true;
        }
        else
        {
            Direction = 0;
            next_Direction = 1;
        }
    }
}

// routine to communicate between device and GUI
// reads GPIOs and updates the GUI
// Used for both chips in Stepper mode (not used in brushed mode)
void UpdateGUIControls_DRV8845()
{
    // Disable Stepper Motors
    if ((gDRV884x_Obj.gDRVEN1 == 0 || gDRV884x_Obj.gDRVEN2 == 0)  && gDRV884x_Obj.nSleep == 1)
    {
        if (RunStepper == true || ManualAdvance == true || Reciprocate == true)
        {
            // disable motor operation
            if (gDRV884x_Obj.gDRVEN1 == 0)
            {
                if (gStepper_Obj1.RUN_STEPPER)
                gStepper_Obj1.RUN_STEPPER   = false;

                if (gStepper_Obj1.MANUAL_ADVANCE)
                gStepper_Obj1.MANUAL_ADVANCE   = false;
            }

            if (gDRV884x_Obj.gDRVEN2 == 0)
            {
                if (gStepper_Obj2.RUN_STEPPER)
                gStepper_Obj2.RUN_STEPPER   = false;

                if (gStepper_Obj2.MANUAL_ADVANCE)
                gStepper_Obj2.MANUAL_ADVANCE   = false;
            }

            if (gDRV884x_Obj.gDRVEN1 == 0 && gDRV884x_Obj.gDRVEN2 == 0)
            {
                RunStepper                      = false;
                ManualAdvance                   = false;
                Reciprocate                     = false;
                MOTOR_CMD_ACTIVE1_STEPPER       = false;
                MOTOR_CMD_ACTIVE2_STEPPER       = false;
                MOTOR_CMD_ACTIVE3_STEPPER       = false;

                // Stop the timer
                TA1CTL &= ~MC_3;
                TA1CTL |= TACLR;

                // clear temp_reciprocation and MoveSteps so next movement is correct
                temp_reciprocation = false;                    // reset reciprocation
                MoveSteps = false;                             // clear MoveSteps so next movement is correct
                AccelerateState = IDLE;                        // set AccelerateState to idle when sleeping or disabled
            }
        }
    }
} // End UpdateGUIControls_DRV8845()

// Routine to determine if any changes are in motor operation are required and pass the information to the motor state machine
void UpdateSpeed_8845()
{
    if (gDRV884x_Obj.gDRVEN1 && !gDRV884x_Obj.gDRVEN2)               // Only stepper 1 is enabled
    {
        // Stepper 1 settings
        StartingSpeed            = gStepper_Obj1.STARTING_SPEED;     // Configure the starting speed
        StoppingSpeed            = gStepper_Obj1.STOPPING_SPEED;     // Configure the stopping speed
        StepsToMove              = gStepper_Obj1.NUM_STEPS;          // Configure the number of steps
        StepsToStop              = gStepper_Obj1.STEPS_2_STOP;       // Configure the steps to stop
        DesiredTargetSpeed       = gStepper_Obj1.TARGET_SPEED;       // Configure the target speed
        AccelRate                = gStepper_Obj1.ACCEL_RATE;         // Configure the acceleration rate here
        StepMode                 = gDRV884x_Obj.StepMode1;           // Configure the step mode
        Reciprocate              = gStepper_Obj1.RECIPROCATION;      // Reciprocation button status
        Direction                = gStepper_Obj1.gDIR;               // Direction arrows status
        ManualAdvance            = gStepper_Obj1.MANUAL_ADVANCE;     // Manual advance button status
        RunStepper               = gStepper_Obj1.RUN_STEPPER;        // Start Input based Steps button status
        ichop_val                = gBrushed_Obj.VREF1;               // Current setting ichop value
    }
    else if (!gDRV884x_Obj.gDRVEN1 && gDRV884x_Obj.gDRVEN2)          // Only stepper 2 is enabled
    {
        // Stepper 2 settings
        StartingSpeed            = gStepper_Obj2.STARTING_SPEED;     // Configure the starting speed
        StoppingSpeed            = gStepper_Obj2.STOPPING_SPEED;     // Configure the stopping speed
        StepsToMove              = gStepper_Obj2.NUM_STEPS;          // Configure the number of steps
        StepsToStop              = gStepper_Obj2.STEPS_2_STOP;       // Configure the steps to stop
        DesiredTargetSpeed       = gStepper_Obj2.TARGET_SPEED;       // Configure the target speed
        AccelRate                = gStepper_Obj2.ACCEL_RATE;         // Configure the acceleration rate here
        StepMode                 = gDRV884x_Obj.StepMode2;           // Configure the step mode
        Reciprocate              = gStepper_Obj2.RECIPROCATION;      // Reciprocation button status
        Direction                = gStepper_Obj2.gDIR;               // Direction arrows status
        ManualAdvance            = gStepper_Obj2.MANUAL_ADVANCE;     // Manual advance button status
        RunStepper               = gStepper_Obj2.RUN_STEPPER;        // Start Input based Steps button status
        ichop_val                = gBrushed_Obj.VREF2;               // Current setting ichop value
    }
    else if (gDRV884x_Obj.gDRVEN1 && gDRV884x_Obj.gDRVEN2)           // Stepper 1 and 2 enabled
    {
        gStepper_Obj2.STARTING_SPEED   = gStepper_Obj1.STARTING_SPEED;
        gStepper_Obj2.STOPPING_SPEED   = gStepper_Obj1.STOPPING_SPEED;
        gStepper_Obj2.NUM_STEPS        = gStepper_Obj1.NUM_STEPS;
        gStepper_Obj2.STEPS_2_STOP     = gStepper_Obj1.STEPS_2_STOP;
        gStepper_Obj2.TARGET_SPEED     = gStepper_Obj1.TARGET_SPEED;
        gStepper_Obj2.ACCEL_RATE       = gStepper_Obj1.ACCEL_RATE;
        gDRV884x_Obj.StepMode2         = gDRV884x_Obj.StepMode1;
        gStepper_Obj2.RECIPROCATION    = gStepper_Obj1.RECIPROCATION;\
        gStepper_Obj2.gDIR             = gStepper_Obj1.gDIR;
        gStepper_Obj2.MANUAL_ADVANCE   = gStepper_Obj1.MANUAL_ADVANCE;
        gStepper_Obj2.RUN_STEPPER      = gStepper_Obj1.RUN_STEPPER;
        gBrushed_Obj.VREF2             = gBrushed_Obj.VREF1;

        // Steppers settings
        StartingSpeed            = gStepper_Obj1.STARTING_SPEED;     // Configure the starting speed
        StoppingSpeed            = gStepper_Obj1.STOPPING_SPEED;     // Configure the stopping speed
        StepsToMove              = gStepper_Obj1.NUM_STEPS;          // Configure the number of steps
        StepsToStop              = gStepper_Obj1.STEPS_2_STOP;       // Configure the steps to stop
        DesiredTargetSpeed       = gStepper_Obj1.TARGET_SPEED;       // Configure the target speed
        AccelRate                = gStepper_Obj1.ACCEL_RATE;         // Configure the acceleration rate here
        StepMode                 = gDRV884x_Obj.StepMode1;           // Configure the step mode
        Reciprocate              = gStepper_Obj1.RECIPROCATION;      // Reciprocation button status
        Direction                = gStepper_Obj1.gDIR;               // Direction arrows status
        ManualAdvance            = gStepper_Obj1.MANUAL_ADVANCE;     // Manual advance button status
        RunStepper               = gStepper_Obj1.RUN_STEPPER;        // Start Input based Steps button status
        ichop_val                = gBrushed_Obj.VREF1;               // Current setting ichop value
    }

	DetermineMotorMovement_8845();	                                 // Type of motor movement is determined
	MotorStateMachine_8845();	                                     // Command the motor to move
}

// Routine to determine desired motor operation with direction, acceleration, deceleration,
// steps to move and reciprocation
void DetermineMotorMovement_8845()
{
	// calculate steps to target and steps to stop prior to starting motor
	if ((RunStepper == true || ManualAdvance == true
			|| Reciprocate == true) && MOTOR_CMD_ACTIVE1_STEPPER == false
			&& MOTOR_CMD_ACTIVE2_STEPPER == false && MOTOR_CMD_ACTIVE3_STEPPER == false)
	{
		// update steps to speed and steps to stop calculations
		// only calculate when the motor is idle and commanded to move
		time = (float) (DesiredTargetSpeed - StartingSpeed) / AccelRate;
		time_squared = time * time;

		// calculate steps to target
		Steps2Target = ((AccelRate >> 1) * (time_squared)) + (StartingSpeed * time);

		// calculate Steps2Stop if StepsToStop in GUI is >1 and StoppingSpeed is not equal Starting Speed
		if ((StepsToStop == 2 && use_one_edge == 1) || StepsToStop == 1)
		{
			if (StartingSpeed != StoppingSpeed)
			{	// recalculate time if necessary
				time = (float) (DesiredTargetSpeed - StoppingSpeed) / AccelRate;
				time_squared = time * time;
			}
			// calculate steps required to stop
			Steps2Stop = ((AccelRate >> 1) * (time_squared))
					+ (StartingSpeed * time);
		} 
		else			                                       // defined number of steps to stop
			Steps2Stop = StepsToStop;

		// calculate stepping rate update (integer value divide by 32)
		if (AccelRate > 31 || AccelRate == 0)	               // if >=32 or 0
			SteppingRateUpdate = AccelRate >> 5;	           // divide by 32 - right shift 5
		else 
			SteppingRateUpdate = 1;	                           // increment by 1 to get correct value

		if ((StartingSpeed >= DesiredTargetSpeed) || (StoppingSpeed >= DesiredTargetSpeed))
				// Special case -- override prior settings allow motor to attempt to run only at starting speed
		{
			SteppingRateUpdate = 0;
			Steps2Target = 0;
			Steps2Stop = 0;
		}

		// transition from ACCEL to DECEL when count equals (StepsToMove - Steps2Stop) <=0
		if ((Steps2Stop > (StepsToMove >> 1)) && (StartingSpeed == StoppingSpeed))
			Steps2Decel = StepsToMove >> 1;	// decel at midpoint
		else
			Steps2Decel = StepsToMove - Steps2Stop;
	}

	// start running or stopping the motor if commanded and not manually advancing
	if (RunStepper == true && MOTOR_CMD_ACTIVE1_STEPPER == false
			&& MOTOR_CMD_ACTIVE2_STEPPER == false && MOTOR_CMD_ACTIVE3_STEPPER == false)
	{
		// START STEPPER
	    MOTOR_CMD_ACTIVE1_STEPPER = true;		           // Indicate that the motor is  running
		LastCommandWasRunSteps = true;	                   // flag to determine if the timer requires clearing
		DesiredStepperSpeed = DesiredTargetSpeed;	       // Configure the final stepping rate in PPS
		SteppingRate = StartingSpeed;	                   // This is the starting speed from  the GUI adjusted by the edge indicator
		SteppingRateTMR = SMCLK_8845 / StartingSpeed;	   // Convert here to the timer value -- xxTMR means the value to be loaded into the timer
		AccelerateState = ACCEL;		                   // signal to start state machine
		// Configure TimerA
		TA1CCTL2 &= 0xFF1F;				                   // Clear OUTMODx bits; 3 MSB on lower byte; Timer configured to output mode
		TA1CCTL2 |= OUTMOD_4 + CCIE;	                   // Configure the timer as TOGGLE. TBCCR0 does the reset
		TA1CCR0 = SteppingRateTMR;		                   // Start the timer here with initial value
		TA1CCR2 = SteppingRateTMR >> 1;	                   // set TBCCR1 to the midpoint of the TBCCR0 for toggle configure TimerB

		// configure Timer B0
		TB0CCTL0 &= 0xFF1F;				                   // Clear OUTMODx bits; 3 MSB on lower byte; Timer configured to output mode
		TB0CCTL0 |= OUTMOD_4 + CCIE;
		TB0CCR0 = TIME_32ms45;		                       // set timer to 32ms
		TB0CCR1 = TIME_32ms45 >> 1;
		
		// start timers
		TA1CTL |= MC_1;		                               // Start timerA by selecting up mode
		TB0CTL |= MC_1;		                               // start timerB
	}

    // motor is running and stop is issued
	if ((RunStepper == false && MOTOR_CMD_ACTIVE1_STEPPER == true))
	{
		// STOP STEPPER
	    MOTOR_CMD_ACTIVE1_STEPPER = false;	               // must set low when stopping
		AccelerateState = DECEL;
		DesiredStepperSpeed = StoppingSpeed;	           // Speed to stop at
	}

	// MOVE_STEPS, Manually advance steps if commanded. Only allow starting if not automatically running
	if (ManualAdvance == true && MOTOR_CMD_ACTIVE1_STEPPER == false
			&& MOTOR_CMD_ACTIVE2_STEPPER == false && MOTOR_CMD_ACTIVE3_STEPPER == false)
	{
	    MOTOR_CMD_ACTIVE2_STEPPER = true;

		// Start Stop
		if (LastCommandWasRunSteps == true || use_one_edge == true)
		{
			// reset timer only when starting or when using one edge
			TA1CTL = TASSEL_2 | ID_3 | MC_0 | TACLR;       // Reconfigure Timer A1
			TA1CCTL2 &= 0xFF1F;	                           // Clear OUTMODx bits; 3 MSB on lower byte; Timer configured to output mode
			TA1CCTL2 |= OUTMOD_4 + CCIE;	               // Configure the timer as SET_RESET. TBCCR0 does the reset
		}

		LastCommandWasRunSteps = false;	                   // reset flag to indicate move steps was last commanded
		DesiredStepperSpeed = DesiredTargetSpeed;		   // Configure the Frequency Rate

		SteppingRate = StartingSpeed;
		SteppingRateTMR = SMCLK_8845 / StartingSpeed;	   // convert to timer value
		AccelerateState = ACCEL;		                   // signal to start state machine
		tmpStepsToMove = 0;				                   // clear counter
		MoveSteps = true;				                   // set flag to indicate manual advance
		
		// configure Timer A
		TA1CCTL2 &= 0xFF1F;                                // Clear OUTMODx bits; 3 MSB on lower byte; Timer configured to output mode
		TA1CCTL2 |= OUTMOD_4 + CCIE;                       // Configure the timer as TOGGLE. TBCCR0 does the reset
		TA1CCR0 = SteppingRateTMR;		                   // start the timer
		TA1CCR2 = SteppingRateTMR >> 1;	                   // now using count up
		
		// configure TimerB
		TBCCTL0 &= 0xFF1F;				                   // Clear OUTMODx bits; 3 MSB on lower byte; Timer configured to output mode
		TBCCTL0 |= OUTMOD_4 + CCIE;
		TBCCR0 = TIME_32ms45;			                   // set timer to 32ms
		TBCCR1 = TIME_32ms45 >> 1;
		
		// start timers
		TA1CTL |= MC_1;		                               // Start timers by selecting up mode
		TB0CTL |= MC_1;
	}

	if (Reciprocate == true && temp_reciprocation == false)
	{
		last_reciprocation_state = 1;
		temp_reciprocation = true;
		MOTOR_CMD_ACTIVE3_STEPPER = true;

        if (Reciprocate == true)
        {
            __delay_cycles(2000000);
            if (Direction == 1)                            // change direction when done with the number of steps in one direction
            {
                Direction = 0;
                if (gStepper_Obj1.MANUAL_ADVANCE)
                {
                    if (gStepper_Obj1.gDIR == 1)
                        gStepper_Obj1.gDIR = 0;
                }

               if (gStepper_Obj2.MANUAL_ADVANCE)
                {
                    if (gStepper_Obj2.gDIR == 1)
                        gStepper_Obj2.gDIR = 0;
                }
            } else
            {
                Direction = 1;
                if (gStepper_Obj1.MANUAL_ADVANCE)
                {
                    if (gStepper_Obj1.gDIR == 0)
                        gStepper_Obj1.gDIR = 1;
                }

                if (gStepper_Obj2.MANUAL_ADVANCE)
                {
                    if (gStepper_Obj2.gDIR == 0)
                        gStepper_Obj2.gDIR = 1;
                }
            }
        }

        if ((!gStepper_Obj1.MANUAL_ADVANCE && gStepper_Obj1.RECIPROCATION && gDRV884x_Obj.gDRVEN1) ||
                (!gStepper_Obj2.MANUAL_ADVANCE && gStepper_Obj2.RECIPROCATION && gDRV884x_Obj.gDRVEN2))
        {
                Reciprocate = false;
                temp_reciprocation = false;                // reset reciprocation
	    }

        if ((gStepper_Obj1.MANUAL_ADVANCE && gStepper_Obj1.RECIPROCATION) || (gStepper_Obj2.MANUAL_ADVANCE && gStepper_Obj2.RECIPROCATION))
        {
            if (LastCommandWasRunSteps == true || use_one_edge == true)
            {
                // reset timer only when starting or when using one edge
                TA1CTL = TASSEL_2 | ID_3 | MC_0 | TACLR;   // Reconfigure Timer A1
                TA1CCTL2 &= 0xFF1F;                        // Clear OUTMODx bits; 3 MSB on lower byte; Timer configured to output mode
                TA1CCTL2 |= OUTMOD_4 + CCIE;               // Configure the timer as SET_RESET. TBCCR0 does the reset
            }
            LastCommandWasRunSteps = false;                // reset flag to indicate move steps was last commanded
            DesiredStepperSpeed = DesiredTargetSpeed;      // Configure the Frequency Rate

            SteppingRate = StartingSpeed;
            SteppingRateTMR = SMCLK_8845 / StartingSpeed;  // convert to timer value
            AccelerateState = ACCEL;                       // signal to start state machine
            tmpStepsToMove = 0;                            // clear counter
            MoveSteps = true;                              // set flag to indicate manual advance

            // configure Timer A
            TA1CCR0 = SteppingRateTMR;                     // start the timer
            TA1CCR2 = SteppingRateTMR >> 1;                // now using count up

            // configure TimerB
            TBCCTL0 &= 0xFF1F;                             // Clear OUTMODx bits; 3 MSB on lower byte; Timer configured to output mode
            TBCCTL0 |= OUTMOD_4 + CCIE;
            TBCCR0 = TIME_32ms45;                          // set timer to 32ms
            TBCCR1 = TIME_32ms45 >> 1;

            // start timers
            TA1CTL |= MC_1;                                // Start timers by selecting up mode
            TB0CTL |= MC_1;
        }
	}

	// Special case of updating speed during running. If the Desired Target Speed
	// has been updated, then the stepper speed should be increased or decreased
	if ((RunStepper == true) && (DesiredStepperSpeed != DesiredTargetSpeed))
	{
		if (DesiredTargetSpeed > DesiredStepperSpeed)
			AccelerateState = ACCEL;
		else
			AccelerateState = DECEL2NEWTARGET;

		// reset DesiredStepperSpeed
		DesiredStepperSpeed = DesiredTargetSpeed;
	}
}

// Stepper state machine function
void MotorStateMachine_8845()
{
    // update the SteppingRateTMR if required.  Reduce the number of calculations
	if (UpdateSteppingRateTMR == true)
	{
		SteppingRateTMR = SMCLK_8845 / SteppingRate;	   // calculate new value
		UpdateSteppingRateTMR = false;	                   // clear flag
	}

	switch (AccelerateState)
	{
	case (IDLE):
		if (Reciprocate == false && gDRV884x_Obj.nSleep == 1 && (gDRV884x_Obj.gDRVEN1 == 1 || gDRV884x_Obj.gDRVEN2 == 1))
		{
            // Stop
		    MOTOR_CMD_ACTIVE1_STEPPER  = false;
		    MOTOR_CMD_ACTIVE2_STEPPER  = false;
		    MOTOR_CMD_ACTIVE3_STEPPER  = false;
		    MoveSteps                  = false;            // clear the MoveSteps flag here

			RunStepper                 = false;
            if (gStepper_Obj1.RUN_STEPPER)
                gStepper_Obj1.RUN_STEPPER   = false;
            if (gStepper_Obj2.RUN_STEPPER)
                gStepper_Obj2.RUN_STEPPER   = false;

            ManualAdvance              = false;           // clear the flags once stops
            if (gStepper_Obj1.MANUAL_ADVANCE)
                gStepper_Obj1.MANUAL_ADVANCE   = false;
            if (gStepper_Obj2.MANUAL_ADVANCE)
                gStepper_Obj2.MANUAL_ADVANCE   = false;

			// reset the timers if the last command was run motor
			if (LastCommandWasRunSteps == true)
			{
				TA1CTL &= ~MC_3;	                       // Stop Timer A1
				TB0CTL &= ~MC_3;	                       // Stop Timer B0
	        }
		}
		break;
	case (ACCEL):                                          // do nothing in ACCEL mode, wait for external input to change state
		break;
	case (DECEL):                                          // do nothing in DECEL mode, wait for external input to change state
		break;
	case (RUN):                                            // do nothing in RUN mode, wait for external input to change state
		break;
	case (DECEL2NEWTARGET):                                // do nothing in DECEL mode, wait for external input to change state
	    break;
	}
}
