/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  stepper_ISR.h - Declaration file for functions and global variables related
 *  to stepper control
 *  DRV884x_EVM_FIRMWARE
 *  Updated 03/01/2023
 *  Murugavel Raju
 *
 ******************************************************************************/
#include "drv884x/include/drv8845.h"
#include "drv884x/include/utility.h"
#include "msp430.h"
#include "stepper.h"
#include "stepper_8845.h"
#include "intrinsics.h"
#include "Sinewave_values.h"

#pragma vector= DAC12_VECTOR, DMA_VECTOR , USCI_A0_VECTOR, USCI_B0_VECTOR, USCI_B1_VECTOR, \
                PORT1_VECTOR, PORT2_VECTOR, PORT3_VECTOR, PORT4_VECTOR, \
                RTC_VECTOR, COMP_B_VECTOR, LDO_PWR_VECTOR, \
                TIMER0_A1_VECTOR, TIMER0_B1_VECTOR, \
                TIMER2_A1_VECTOR, UNMI_VECTOR, SYSNMI_VECTOR

__interrupt void Trap_ISR(void)
{
}

// Timer A0:
// DRV8849 Stepper 1 STEP output. Stepper Motor 1 with STEP/DIR control. STEP1 is on P1.2, so TA0CCR1.
#pragma vector=TIMER0_A0_VECTOR
__interrupt void Timer0_A0(void)
{
    if (gDRV884x_Obj.deviceID == 8849)                     // Interrupt source was DRV8849
    {
        // Reload the timers
        TA0CCR0 = SteppingRateTMR1;                        // set STEP period
        TA0CCR1 = SteppingRateTMR1 >> 1;                   // set at 50% duty cycle
        UpdateNextAction1 = true;

        // check to stop timers if counting steps
        if (MoveSteps1)                                    // if manual advance count steps
        {                                                  // when counts completed
            tmpStepsToMove1++;
            // stop interrupts here to avoid latency
            if (tmpStepsToMove1 >= StepsToMove1)           // stop here when equal
            {
                TA0CTL &= ~MC_3;                           // Stop timerA0 (Stepper1 STEP output)
                TA2CTL &= ~MC_3;                           // Stop timerA2 (Stepper1 Accel/decel)
            }
        }
    } else                                                 // Interrupt source was DRV8845 Brushed DC motor mode
    {
        __bic_SR_register_on_exit(LPM0_bits);              // Clear LPM0 bits (Low Power Mode 0)
    }

}

// Timer A2: 
// DRV8849 Stepper 1 acceleration and deceleration control
#pragma vector=TIMER2_A0_VECTOR
__interrupt void Timer2_A0(void)
{
    enable_Stepper1Timer_Interrupt = true;

    // disable interrupts if target speed is reached
    if (gStepper_Obj1.AccelerateState == RUN && gStepper_Obj1.RUN_STEPPER == true)
    {
    	// disable Timer_A0 interrupt
    	enable_Stepper1Timer_Interrupt = false;
    }

    UpdateMotorParameters1 = true;
}

// Timer A1:  
// DRV8849 Stepper 2 Acceleration and Deceleration control
#pragma vector=TIMER1_A0_VECTOR
__interrupt void Timer1_A0(void)
{
    enable_Stepper2Timer_Interrupt = true;
    // disable interrupts if target speed is reached
    if (gStepper_Obj2.AccelerateState == RUN && gStepper_Obj2.RUN_STEPPER == true)
    {
    	// disable Timer_B0 interrupt
    	enable_Stepper2Timer_Interrupt = false;
    }

    UpdateMotorParameters2 = true;
}

// Timer A1:
// DRV8845 Stepper 1 main and microstepping control
#pragma vector=TIMER1_A1_VECTOR
__interrupt void Timer1_A1(void)
{
    // DRV8845 Stepper Motor 1 (PHASE/ENABLE control)
    if (gDRV884x_Obj.deviceID == 8845 && gDRV884x_Obj.MotorMode == 0)
    {
    switch (TA1IV)
    {
	case TA1IV_TACCR1:
		break;
	case TA1IV_TACCR2:
		if (MoveSteps)		                               // if manual advance count steps
		{
			tmpStepsToMove++;	                           // increment step counter
			if (tmpStepsToMove >= Steps2Decel)
			{
				AccelerateState = DECEL;	               // start deceleration
			}

			if (tmpStepsToMove > StepsToMove)	           // stop here when greater than to let it run through one more step
			{
				AccelerateState = IDLE;
				temp_reciprocation = false;
				TA1CTL &= ~MC_3;	                       // Stop TimerA
				TB0CTL &= ~MC_3;	                       // Stop TimerB
				MoveSteps = false;
			}

		}

		// If the motor state machine is not idle and move_to_next_step is one, advance the sine wave table
		if (AccelerateState != IDLE)
		{
			// calculate next value and adjust for over/underflow
			if (Direction == 0)
			{
				Next_a_step += step_offset;
				if (Next_a_step > 2047)
					Next_a_step &= 2047;

				Next_b_step += step_offset;
				if (Next_b_step > 2047)
					Next_b_step &= 2047;
			} else
			{
				Next_a_step -= step_offset;
				if (Next_a_step > 2048)
					Next_a_step &= 2047;

				Next_b_step -= step_offset;
				if (Next_b_step > 2048)
					Next_b_step &= 2047;
			}

			// if STEP_MODE has changed, align values to proper steps. This is done by masking the lower values
			if (temp_step_mode > 1)
			{
				Next_a_step &= (0x7ff << (10 - temp_step_mode));
				Next_b_step &= (0x7ff << (10 - temp_step_mode));
			} else if (temp_step_mode == 0)
			{	                                           // align to 200 hex then change to odd
				Next_a_step &= (0x7ff << 9);
				Next_a_step |= BIT8;	                   // align to odd 100 hex
				Next_b_step &= (0x7ff << 9);
				Next_b_step |= BIT8;	                   // align to odd 100 hex
			} else
			{		                                       // align to 100 hex
				Next_a_step &= (0x7ff << 8);
				Next_b_step &= (0x7ff << 8);
			}

			Next_a_sine_table = SineWave[Next_a_step];
			Next_b_sine_table = SineWave[Next_b_step];

			// Create scaling values half, quarter, eight, and sixteenth, then combine as required. For non-circular 1/2 step, use full scale.
			if (temp_step_mode == 1)
				Next_full_a = 0xfff;
			else
				Next_full_a = Next_a_sine_table & Lower_12_bits;

			Next_half_a = Next_full_a >> 1;
			Next_quarter_a = Next_full_a >> 2;
			Next_eighth_a = Next_full_a >> 3;
			Next_sixteenth_a = Next_full_a >> 4;

			// Create scaling values half, quarter, eight, and sixteenth, then combine as required. For non-circular 1/2 step, use full scale.
			if (temp_step_mode == 1)
				Next_full_b = 0xfff;
			else
				Next_full_b = Next_b_sine_table & Lower_12_bits;

			Next_half_b = Next_full_b >> 1;
			Next_quarter_b = Next_full_b >> 2;
			Next_eighth_b = Next_full_b >> 3;
			Next_sixteenth_b = Next_full_b >> 4;

			// determine PHA/PHB setting
			Next_PHA = Next_a_sine_table & BITD;
			Next_PHB = Next_b_sine_table & BITD;

			// Modify the values based on GUI Full Scale Current selection
			switch (tmp_ichop_val)
			{
			case 0:		                                             // 100 % full scale
				Next_a_value = Next_full_a;
				Next_b_value = Next_full_b;
				break;
			case 1:		                                             // 93.75 %
				Next_a_value = Next_full_a - Next_sixteenth_a;
				Next_b_value = Next_full_b - Next_sixteenth_b;
				break;
			case 2:		                                             // 87.5 %
				Next_a_value = Next_full_a - Next_eighth_a;
				Next_b_value = Next_full_b - Next_eighth_b;
				break;
			case 3:		                                             // 81.25 %
				Next_a_value = Next_full_a - Next_eighth_a - Next_sixteenth_a;
				Next_b_value = Next_full_b - Next_eighth_b - Next_sixteenth_b;
				break;
			case 4:		                                             // 75 %
				Next_a_value = Next_full_a - Next_quarter_a;
				Next_b_value = Next_full_b - Next_quarter_b;
				break;
			case 5:		                                             // 68.75 %
				Next_a_value = Next_full_a - Next_quarter_a - Next_sixteenth_a;
				Next_b_value = Next_full_b - Next_quarter_b - Next_sixteenth_b;
				break;
			case 6:		                                             // 62.5 %
				Next_a_value = Next_half_a + Next_eighth_a;
				Next_b_value = Next_half_b + Next_eighth_b;
				break;
			case 7:		                                             // 56.25 %
				Next_a_value = Next_half_a + Next_sixteenth_a;
				Next_b_value = Next_half_b + Next_sixteenth_b;
				break;
			case 8:		                                             // 50 %
				Next_a_value = Next_half_a;
				Next_b_value = Next_half_b;
				break;
			case 9:		                                             // 43.75 %
				Next_a_value = Next_half_a - Next_sixteenth_a;
				Next_b_value = Next_half_b - Next_sixteenth_b;
				break;
			case 10:		                                         // 37.5 %
				Next_a_value = Next_half_a - Next_eighth_a;
				Next_b_value = Next_half_b - Next_eighth_b;
				break;
			case 11:		                                         // 31.25 %
				Next_a_value = Next_quarter_a + Next_sixteenth_a;
				Next_b_value = Next_quarter_b + Next_sixteenth_b;
				break;
			case 12:		                                         // 25 %
				Next_a_value = Next_quarter_a;
				Next_b_value = Next_quarter_b;
				break;
			case 13:		                                         // 18.75 %
				Next_a_value = Next_eighth_a + Next_sixteenth_a;
				Next_b_value = Next_eighth_b + Next_sixteenth_b;
				break;
			case 14:		                                         // 12.5 %
				Next_a_value = Next_eighth_a;
				Next_b_value = Next_eighth_b;
				break;
			case 15:		                                         // 6.25 %
				Next_a_value = Next_sixteenth_a;
				Next_b_value = Next_sixteenth_b;
				break;
            case 16:                                                 // 0 %
                Next_a_value = 0;
                Next_b_value = 0;
                break;
			}

			AVREF_DAC = Next_a_value;	                             // AVREF
			BVREF_DAC = Next_b_value;	                             // BVREF

			// Capture I0x_I1x settings for non-circular 1/2 Step mode
            if (temp_step_mode == 1)
            {
                if ((Next_a_sine_table & BITF) == 0)
                {
                    if (gDRV884x_Obj.gDRVEN1)
                        set_I01_I11_hi;

                    if (gDRV884x_Obj.gDRVEN2)
                        set_I03_I13_hi;

                    if (Next_PHA == 0)
                    {
                        set_phase1_hi;
                        set_phase3_hi;
                    }
                    else
                    {
                        set_phase1_lo;
                        set_phase3_lo;
                    }
                } else
                {
                    if (gDRV884x_Obj.gDRVEN1)
                        set_I01_I11_lo;

                    if (gDRV884x_Obj.gDRVEN2)
                        set_I03_I13_lo;

                    if (Next_PHA == 0)
                    {
                        set_phase1_hi;
                        set_phase3_hi;
                    }
                    else
                    {
                        set_phase1_lo;
                        set_phase3_lo;
                    }
                }

                if ((Next_b_sine_table & BITF) == 0)
                {
                    if (gDRV884x_Obj.gDRVEN1)
                        set_I02_I12_hi;

                    if (gDRV884x_Obj.gDRVEN2)
                        set_I04_I14_hi;

                    if (Next_PHB == 0)
                    {
                        set_phase2_hi;
                        set_phase4_hi;
                    }
                    else
                    {
                        set_phase2_lo;
                        set_phase4_lo;
                    }
                } else
                {
                    if (gDRV884x_Obj.gDRVEN1)
                        set_I02_I12_lo;

                    if (gDRV884x_Obj.gDRVEN2)
                        set_I04_I14_lo;

                    if (Next_PHB == 0)
                    {
                        set_phase2_hi;
                        set_phase4_hi;
                    }
                    else
                    {
                        set_phase2_lo;
                        set_phase4_lo;
                    }
                }
            } else                                                   // set ENA low if Next_a_value = 0 for all other modes
            {
                if (Next_a_value == 0)
                {
                    if (gDRV884x_Obj.gDRVEN1)
                        set_I01_I11_hi;

                    if (gDRV884x_Obj.gDRVEN2)
                        set_I03_I13_hi;
                }
                else
                {
                    if (gDRV884x_Obj.gDRVEN1)
                        set_I01_I11_lo;

                    if (gDRV884x_Obj.gDRVEN2)
                        set_I03_I13_lo;
                }

                if (Next_PHA == 0)
                {
                    set_phase1_hi;
                    set_phase3_hi;
                }
                else
                {
                    set_phase1_lo;
                    set_phase3_lo;
                }

                if (Next_b_value == 0)
                {
                    if (gDRV884x_Obj.gDRVEN1)
                        set_I02_I12_hi;

                    if (gDRV884x_Obj.gDRVEN2)
                        set_I04_I14_hi;
                }
                else
                {
                    if (gDRV884x_Obj.gDRVEN1)
                        set_I02_I12_lo;

                    if (gDRV884x_Obj.gDRVEN2)
                        set_I04_I14_lo;
                }

                if (Next_PHB == 0)
                {
                    set_phase2_hi;
                    set_phase4_hi;
                }
                else
                {
                    set_phase2_lo;
                    set_phase4_lo;
                }
            }
		}

        // Reload the timers
		TA1CCR0 = SteppingRateTMR;
		TA1CCR2 = SteppingRateTMR >> 1;
		break;
	case TA1IV_TAIFG:
		break;
	}
    } // End DRV8845 Stepper Motor 1 with PHASE/ENABLE control
} // End Timer A1:  DRV8849 Stepper 1 STEP output


// Timer B0:
// DRV8845 - Stepper 1 Acceleration and Deceleration control
// DRV8849:  Stepper 2 STEP output.  STEP2 is on P1.2, so TB0CCR1.  
#pragma vector=TIMER0_B0_VECTOR
__interrupt void TimerB0(void)
{
    // DRV8849 - Stepper Motor 2 with STEP/DIR control
    if (gDRV884x_Obj.deviceID == 8849)
    {
        // Reload the timers
        TB0CCR0 = SteppingRateTMR2;                        // set STEP period
        TB0CCR1 = SteppingRateTMR2 >> 1;                   // set at 50% duty cycle
        UpdateNextAction2 = true;
        
        // check to stop timers if counting steps
        if (MoveSteps2)                                    // if manual advance count steps
        {                                                  // when counts completed
            tmpStepsToMove2++;
            // stop interrupts here to avoid latency
            if (tmpStepsToMove2 >= StepsToMove2)           // stop here when equal
            {
                TB0CTL &= ~MC_3;                           // Stop TimerB0 (Stepper2 STEP output)
                TA1CTL &= ~MC_3;                           // Stop TimerA1 (Stepper2 Accel/decel)
            }
        }
    }

    // Update Motor Parameters, increment or decrement to the new Timer B0 values
    if (gDRV884x_Obj.deviceID == 8845)
    {
		if (AccelerateState == ACCEL && (SteppingRate <= DesiredTargetSpeed))
		{
			// increase pps and calculate new pps
			if (SteppingRate + SteppingRateUpdate >= DesiredTargetSpeed)
			{
				AccelerateState = RUN;	                   // No longer accelerate
				SteppingRate = DesiredTargetSpeed;
			} else
			{
				SteppingRate += SteppingRateUpdate;	       // Increment
				AccelerateState = ACCEL;
			}
		}
		if (AccelerateState == DECEL)
		{
			if (SteppingRate >= StoppingSpeed)
			{
				// decrease pps and calculate new pps
				if (SteppingRate - SteppingRateUpdate <= StoppingSpeed)
				{
					if (MoveSteps == false && SteppingRate == StoppingSpeed)
						AccelerateState = IDLE;	           // Stop motor when speed is reached
					SteppingRate = StoppingSpeed;
				} else
				{
					SteppingRate -= SteppingRateUpdate;	   // Decrement
					AccelerateState = DECEL;
				}
			} else		                                   // already less than stopping speed
			{
				if (MoveSteps == false)
					AccelerateState = IDLE;	               // Stop motor when speed is reached
			}
		}

		// this allows for the motor to spin down to a new speed setting without stopping
		if (AccelerateState == DECEL2NEWTARGET)
		{
			if (SteppingRate >= DesiredTargetSpeed)
			{
				// decrease pps and calculate new pps
				if (SteppingRate - SteppingRateUpdate <= DesiredTargetSpeed)
				{
					if (SteppingRate == DesiredTargetSpeed)
						AccelerateState = RUN;	           // switch to RUN when
					// speed is reached
					SteppingRate = DesiredTargetSpeed;
				} else
				{
					SteppingRate -= SteppingRateUpdate;	   // Decrement
					AccelerateState = DECEL2NEWTARGET;
				}
			}
		}

		// Do not perform the division in the interrupt routine. Only update the stepping rate. If the stepping rate has been updated, set a flag
		if (SteppingRate != PriorSteppingRate)
		{
			UpdateSteppingRateTMR = true;	               // Set flag to re-calculate stepping rate TMR
			PriorSteppingRate = SteppingRate;
		}
    }
}
